# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import scipy as SP
import limix.deprecated.io.data_util as du
import pandas as pd

class pheno_reader_tables():
    def __init__(self,file_name):
        self.file_name = file_name
        self.load()

    def load(self,cache_phenotype=True):
        """load data file

        Args:
            cache_phenotype:    load phentopyes fully intro memry (default: True)
        """
        import tables
        self.f = tables.openFile(self.file_name,'r')
        self.pheno = self.f.root.phenotype

        #parse out thse we always need for convenience
        self.pheno_matrix = self.pheno.matrix
        self.sample_ID = self.pheno.row_header.sample_ID[:]
        self.phenotype_ID = self.pheno.col_header.phenotype_ID[:]

        #cache?
        if cache_phenotype:
            self.pheno_matrix = self.pheno_matrix[:]

        # Additional pheno col header
        headers = self.pheno.col_header

        #TODO: create pandas.MultiIndex from headers
        child_names = []#headers._v_children.keys()
        child_arrays = []

        for child in headers:
            if child._v_name=="phenotype_ID":
                pass
                #TODO: OS, I removed this as the whole things otherwise crashes if there are no headers
                #continue
            child_names.append(child._v_name)
            child_arrays.append(child[:])
        multiindex = pd.MultiIndex.from_arrays(arrays=child_arrays,names=child_names)
        self.index_frame = pd.DataFrame(data=SP.arange(self.pheno_matrix.shape[1]),index = multiindex)
        self.headers_frame = pd.DataFrame(data=SP.array(child_arrays).T,index=self.phenotype_ID,columns=child_names)

        if 'gene_ID' in headers:
            self.eqtl = True
            self.gene_ID = self.pheno.col_header.gene_ID[:]
            self.gene_pos_start = SP.array([self.pheno.col_header.gene_chrom[:],self.pheno.col_header.gene_start[:]],dtype='int').T
            self.gene_pos_end = SP.array([self.pheno.col_header.gene_chrom[:],self.pheno.col_header.gene_end[:]],dtype='int').T
            self.gene_ID_list= list(set(self.gene_ID))
        else:
            self.eqtl = False
        if 'environment' in headers:
            self.environment  = self.pheno.col_header.environment[:]
            self.environment_list = list(set(self.environment))
        else:
            self.environment = None

        #dimensions
        self.N = self.pheno_matrix.shape[0]
        self.P = self.pheno_matrix.shape[1]


    def getPhenotypes(self,phenotype_IDs=None,phenotype_query=None,sample_idx=None,center=True,intersection=False):
        """load Phenotypes

        Args:
            phenotype_IDs:      names of phenotypes to load
            phenotype_query:    string hoding a pandas query (e.g. "(environment==1) & (phenotype_ID=='growth')"
                                selects all phenotypes that have a phenotype_ID equal to growth under environment 1.
            sample_idx:         Boolean sample index for subsetting individuals
            center:             Boolean: mean center (and mean-fill in missing values if intersection==False)? (default True)
            impute:             imputation of missing values (default: True)
            intersection:       restrict observation to those obseved in all phenotypes? (default: False)

        Returns:
            phenotypes:     [N x P] scipy.array of phenotype values for P phenotypes
            sample_idx_intersect:        index of individuals in phenotypes after filtering missing valuesS
        """
        if phenotype_IDs is not None:
            I = SP.array([SP.nonzero(self.phenotype_ID==n)[0][0] for n in phenotype_IDs])
        elif phenotype_query is not None:
            try:
                I = self.index_frame.query(phenotype_query).values[:,0]

                #if there are no results we won't actually get an exception, we just get an
                #empty response
                if len(I) == 0:
                    print "query '%s' yielded no results!" % (phenotype_query)
                    I = SP.zeros([0],dtype="int")

            except Exception, arg:

                print "query '%s' yielded no results: %s" % (phenotype_query, str(arg))

                I = SP.zeros([0],dtype="int")
        else:
            I = SP.arange(self.phenotype_ID.shape[0])
        phenotypes = SP.array(self.pheno_matrix[:,I],dtype='float')
        phenotypes = phenotypes[sample_idx]
        Iok = (~SP.isnan(phenotypes))

        if intersection:
            sample_idx_intersect = Iok.all(axis=1)
        else:
            sample_idx_intersect = Iok.any(axis=1)

        phenotypes = phenotypes[sample_idx_intersect]
        Iok = Iok[sample_idx_intersect]

        if center:
            for i in xrange(phenotypes.shape[1]):
                ym = phenotypes[Iok[:,i],i].mean()
                phenotypes[:,i] -= ym
                phenotypes[~Iok[:,i],i] = ym
                phenotypes[:,i] /= phenotypes[:,i].std()
        phenotypes = pd.DataFrame(data=phenotypes, index=self.sample_ID[sample_idx_intersect],columns=self.phenotype_ID[I])
        #calculate overlap of missing values
        return phenotypes, sample_idx_intersect

    def get_pos(self,phenotype_query=None):
        """
        get the positions of the genotypes

        Returns:
            chromosome
            position
            cumulative_position
        """
        assert self.eqtl == True, 'Only for eqtl data'
        if phenotype_query is not None:
            try:
                I = self.index_frame.query(phenotype_query).values[:,0]
            except Exception, arg:

                print "query '%s' yielded no results: %s"%phenotype_query, str(arg)

                I = SP.zeros([0],dtype="int")
            return {"start" : self.gene_pos_start[I], "end" : self.gene_pos_end[I]}
        else:
            return {"start" : self.gene_pos_start, "end" : self.gene_pos_end}


class pheno_reader_h5py_deprecated():
    def __init__(self,file_name):
        self.file_name = file_name
        self.load()

    def load(self,cache_phenotype=True):
        """load data file

        Args:
            cache_phenotype:    load phentopyes fully intro memry (default: True)
        """
        import h5py
        self.f = h5py.File(self.file_name,'r')
        self.pheno = self.f['phenotype']

        #parse out thse we always need for convenience
        self.pheno_matrix = self.pheno['matrix']
        self.sample_ID = self.pheno['row_header']['sample_ID'][:]
        self.phenotype_ID = self.pheno['col_header']['phenotype_ID'][:]

        #cache?
        if cache_phenotype:
            self.pheno_matrix = self.pheno_matrix[:]

        # Additional pheno col header
        headers = self.pheno['col_header'].keys()
        if 'gene_ID' in headers:
            self.eqtl = True
            self.gene_ID = self.pheno['col_header']['gene_ID'][:]
            self.gene_pos = SP.array([self.pheno['col_header']['gene_chrom'][:],self.pheno['col_header']['gene_start'][:],self.pheno['col_header']['gene_end']],dtype='int').T
            self.gene_ID_list= list(set(self.gene_ID))
        else:
            self.eqtl = False
        if 'environment' in headers:
            self.environment  = self.pheno['col_header/environment'][:]
            self.environment_list = list(set(self.environment))
        else:
            self.environment = None

        #dimensions
        self.N = self.pheno_matrix.shape[0]
        self.P = self.pheno_matrix.shape[1]
        #self.pheno_df = pd.DataFrame(data=self.pheno_matrix,index=self.sample_ID,columns=self.phenotype_ID)

    def getPhenotypes_df(self,sample_idx=None,phenotype_IDs=None,center=True,impute=True,intersection=False):
        """load Phenotypes

        Args:
            phenotype_IDs:  names of phenotypes to load
            impute:         imputation of missing values (default: True)
            intersection:   restrict observation to those obseved in all phenotypes? (default: False)

        Returns:
            phenotypes:     [N x P] scipy.array of phenotype values for P phenotypes
            sample_idx_intersect:        index of individuals in phenotypes after filtering missing valuesS
        """
        if phenotype_IDs is not None:
            I = SP.array([SP.nonzero(self.phenotype_ID==n)[0][0] for n in phenotype_IDs])
        else:
            I = SP.arange(self.phenotype_ID.shape[0])
        phenotypes = SP.array(self.pheno_matrix[:,I],dtype='float')
        phenotypes = phenotypes[sample_idx]
        Iok = (~SP.isnan(phenotypes))

        if intersection:
            sample_idx_intersect = Iok.all(axis=1)
        else:
            sample_idx_intersect = Iok.any(axis=1)

        phenotypes = phenotypes[sample_idx_intersect]
        Iok = Iok[sample_idx_intersect]

        if center | impute:
            for i in xrange(phenotypes.shape[1]):
                ym = phenotypes[Iok[:,i],i].mean()
                phenotypes[:,i] -= ym
                phenotypes[~Iok[:,i],i] = ym
                phenotypes[:,i] /= phenotypes[:,i].std()
        #calculate overlap of missing values
        return phenotypes, sample_idx_intersect

    def getPhenotypes(self,sample_idx=None,phenotype_IDs=None,center=True,impute=True,intersection=False):
        """load Phenotypes

        Args:
            phenotype_IDs:  names of phenotypes to load
            impute:         imputation of missing values (default: True)
            intersection:   restrict observation to those obseved in all phenotypes? (default: False)

        Returns:
            phenotypes:     [N x P] scipy.array of phenotype values for P phenotypes
            sample_idx_intersect:        index of individuals in phenotypes after filtering missing valuesS
        """
        if phenotype_IDs is not None:
            I = SP.array([SP.nonzero(self.phenotype_ID==n)[0][0] for n in phenotype_IDs])
        else:
            I = SP.arange(self.phenotype_ID.shape[0])
        phenotypes = SP.array(self.pheno_matrix[:,I],dtype='float')
        phenotypes = phenotypes[sample_idx]
        Iok = (~SP.isnan(phenotypes))

        if intersection:
            sample_idx_intersect = Iok.all(axis=1)
        else:
            sample_idx_intersect = Iok.any(axis=1)

        phenotypes = phenotypes[sample_idx_intersect]
        Iok = Iok[sample_idx_intersect]

        if center | impute:
            for i in xrange(phenotypes.shape[1]):
                ym = phenotypes[Iok[:,i],i].mean()
                phenotypes[:,i] -= ym
                phenotypes[~Iok[:,i],i] = ym
                phenotypes[:,i] /= phenotypes[:,i].std()
        #calculate overlap of missing values
        return phenotypes, sample_idx_intersect



# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import scipy as sp
import copy

import limix.deprecated.io.data_util as du
import limix.deprecated.io.genotype_reader as gr
import limix.deprecated.io.phenotype_reader as pr
import pandas as pd

class QTLData(object):
    """
    This class is a mask to a the contents of a geno_reader and a pheno_reader or a subset thereof
    """

    def __init__(self,geno_reader=None,pheno_reader=None):
        self.geno_reader = geno_reader
        self.pheno_reader = pheno_reader
        #self.geno_pos["chrom"] = self.geno_reader.geno_chrom
        self.geno_pos = self.geno_reader.position
        self.geno_ID = self.geno_reader.geno_ID
        self.phenotype_ID = self.pheno_reader.phenotype_ID
        self.geno_snp_idx = None      #SNP indices
        self.sample_idx = du.merge_indices([self.geno_reader.sample_ID, self.pheno_reader.sample_ID],header=["geno","pheno"],join="inner")      #index of individuals
        self.sample_ID = self.geno_reader.sample_ID[ sp.array(self.sample_idx["geno"])]
        self.num_snps = self.geno_reader.num_snps


    def range_query_geno_local(self, idx_start=None, idx_end=None, chrom=None,pos_start=None, pos_end=None,windowsize=0):
        """
        return an index for a range query on the genotypes
        """
        if idx_start==None and idx_end==None and pos_start==None and pos_end==None and chrom==None:
            return  sp.arange(0,self.num_snps)
        elif idx_start is not None or idx_end is not None:
            if idx_start is None:
                idx_start = 0
            if idx_end is None:
                idx_end = self.num_snps
            res =  sp.arange(idx_start,idx_end)
            return res
        elif chrom is not None:
            res = self.geno_pos["chrom"]==chrom
        elif pos_start is not None or pos_end is not None:
            if pos_start is not None and pos_end is not None:
                assert pos_start[0] == pos_end[0], "chromosomes have to match"

            if pos_start is None:
                idx_larger =  sp.ones(self.num_snps,dtype=bool)
            else:
                idx_larger = (self.geno_pos["pos"]>=(pos_start[1]-windowsize)) & (self.geno_pos["chrom"]==pos_start[0])
            if pos_end is None:
                idx_smaller =  sp.ones(self.num_snps,dtype=bool)
            else:
                idx_smaller = (self.geno_pos["pos"]<=(pos_end[1]+windowsize)) & (self.geno_pos["chrom"]==pos_end[0])
            res = idx_smaller & idx_larger
        else:
            raise Exception("This should not be triggered")#res =  sp.ones(self.geno_pos.shape,dtype=bool)
        return  sp.where(res)[0]


    def range_query_geno(self, idx_start=None, idx_end=None, chrom=None, pos_start=None, pos_end=None,windowsize=0):
        """
        return an index for a range query on the genotypes
        """
        if idx_start==None and idx_end==None and pos_start==None and pos_end==None:
            return self.geno_snp_idx
        else:
            res = self.range_query_geno_local(idx_start=idx_start, idx_end=idx_end, chrom=chrom, pos_start=pos_start, pos_end=pos_end,windowsize=windowsize)
        if self.geno_snp_idx is None:
            return  sp.where(res)[0]
        else:
            return self.geno_snp_idx[res]


    def getGenotypes(self,idx_start=None,idx_end=None,pos_start=None,pos_end=None,windowsize=0,chrom=None,center=True,unit=True,impute_missing=False,cast_float=True):
        """return genotypes.
        Optionally the indices for loading subgroups the genotypes for all people
        can be given in one out of three ways:
        - 0-based indexing (idx_start-idx_end)
        - position (pos_start-pos_end on chrom)
        - cumulative position (pos_cum_start-pos_cum_end)
        If all these are None (default), then all genotypes are returned

        Args:
            idx_start:         genotype index based selection (start index)
            idx_end:         genotype index based selection (end index)
            pos_start:       position based selection (start position)
            pos_end:       position based selection (end position)
            chrom:      position based selection (chromosome)
            pos_cum_start:   cumulative position based selection (start position)
            pos_cum_end:   cumulative position based selection (end position)
            impute_missing: Boolean indicator variable if missing values should be imputed
            cast_float: Boolean indicator variable if output genotypes should be casted as float
        Returns:
            X:          scipy.array of genotype values
        """
        query_idx = self.range_query_geno(idx_start=idx_start, idx_end=idx_end, chrom=chrom, pos_start=pos_start,windowsize=windowsize)
        X = self.geno_reader.getGenotypes(sample_idx= sp.array(self.sample_idx["geno"]),snp_idx=query_idx)
        if impute_missing:
            X = du.imputeMissing(X,center=center,unit=unit)
        if cast_float:
            if X.dtype!='float64':
                X = sp.array(X,dtype='float64')
        return X

    def getCovariance(self,normalize=True,idx_start=None,idx_end=None,pos_start=None,pos_end=None,windowsize=0,chrom=None,center=True,unit=True,blocksize=None,X=None,**kw_args):
        """calculate the empirical genotype covariance in a region"""
        return self.geno_reader.getCovariance(sample_idx= sp.array(self.sample_idx["geno"]),normalize=normalize,idx_start=idx_start,idx_end=idx_end,pos_start=pos_start,pos_end=pos_end,chrom=chrom,center=center,unit=unit,windowsize=windowsize,blocksize=blocksize,X=X,**kw_args)

    def getGenoID(self,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,windowsize=0):
        """get genotype IDs.
        Optionally the indices for loading subgroups the genotype IDs for all people
        can be given in one out of three ways:
        - 0-based indexing (idx_start-idx_end)
        - position (pos_start-pos_end on chrom)
        - cumulative position (pos_cum_start-pos_cum_end)
        If all these are None (default), then all genotypes are returned

        Args:
            idx_start:         genotype index based selection (start index)
            idx_end:         genotype index based selection (end index)
            pos_start:       position based selection (start position)
            pos_end:       position based selection (end position)
            chrom:      position based selection (chromosome)
            pos_cum_start:   cumulative position based selection (start position)
            pos_cum_end:   cumulative position based selection (end position)

        Returns:
            ID:         scipy.array of genotype IDs (e.g. rs IDs)
        """
        query_idx = self.range_query_geno_local(idx_start=idx_start, idx_end=idx_end, chrom=chrom, pos_start=pos_start, pos_end=pos_end, windowsize=windowsize)
        if query_idx is None:
            return self.geno_ID
        else:
            return self.geno_ID[query_idx]



    def getPhenotypes(self,phenotype_IDs=None,phenotype_query=None,center=True,intersection=False):
        """load Phenotypes

        Args:
            idx_start:      phenotype indices to load (start individual index)
            idx_end:       phenotype indices to load (end individual index)
            phenotype_IDs:  names of phenotypes to load
            impute:         imputation of missing values (default: True)
            intersection:   restrict observation to those obseved in all phenotypes (true) or at least in one phenotype (false)? (default: False)

        Returns:
            phenotypes:     phenotype values
            sample_idx_intersect:        index of individuals in phenotypes after filtering missing values
        """

        phenotypes, sample_idx_intersect = self.pheno_reader.getPhenotypes(sample_idx=sp.array(self.sample_idx["pheno"]),phenotype_IDs=phenotype_IDs,phenotype_query=phenotype_query,center=center,intersection=intersection)
        return phenotypes, sample_idx_intersect

    def getPos(self,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,windowsize=0.0):
        """
        get the positions of the genotypes

        Returns:
            chromosome
            position
            cumulative_position
        """
        query_idx = self.range_query_geno_local(idx_start=idx_start, idx_end=idx_end, chrom=chrom, pos_start=pos_start, pos_end=pos_end, windowsize=windowsize)
        if query_idx is None:
            return self.geno_pos
        else:
            return self.geno_pos.iloc[query_idx]

    def subsample(self,rows=None,cols_pheno=None,cols_geno=None,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,windowsize=0.0):
        """sample a particular set of individuals (rows) or phenotypes (cols_pheno) or genotypes (cols_geno)

        Args:
            rows:           indices for a set of individuals
            cols_pheno:     indices for a set of phenotypes
            cols_geno:      indices for a set of SNPs

        Returns:
            QTLdata object holding the specified subset of the data
        """
        if not (idx_start==None and idx_end==None and pos_start==None and pos_end==None and chrom==None):
            query_idx = self.range_query_geno_local(idx_start=idx_start, idx_end=idx_end, chrom=chrom, pos_start=pos_start, pos_end=pos_end, windowsize=windowsize)
            return self.subsample(rows=rows,cols_pheno=cols_pheno,cols_geno=query_idx,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None)
        C = copy.copy(self)

        if rows is not None:
            C.sample_ID = C.sample_ID[rows]         #IDs of individuals
            C.sample_idx = C.sample_idx.iloc[rows]  #index of individuals

        if cols_geno is not None:
            assert cols_geno.dtype=="int"
            C.geno_pos = C.geno_pos.iloc[cols_geno]
            C.geno_ID = C.geno_ID[cols_geno]
            if C.geno_snp_idx is not None:
                C.geno_snp_idx = C.geno_snp_idx[cols_geno]
            else:
                 C.geno_snp_idx = cols_geno
            C.num_snps=len(C.geno_snp_idx)

        if cols_pheno is not None:
            C.phenotype_ID = C.phenotype_ID[cols_pheno]

        return C

    def subsample_phenotypes(self,phenotype_IDs=None,phenotype_query=None,center=True,intersection=False):
        """load Phenotypes

        Args:
            idx_start:      phenotype indices to load (start individual index)
            idx_end:       phenotype indices to load (end individual index)
            phenotype_IDs:  names of phenotypes to load
            center:         imputation of missing values (default: True)
            intersection:   restrict observation to those obseved in all phenotypes (true) or at least in one phenotype (false)? (default: False)

        Returns:
            phenotypes:     phenotype values
            sample_idx_intersect:        index of individuals in phenotypes after filtering missing values
        """

        phenotypes, sample_idx_intersect = self.pheno_reader.getPhenotypes(phenotype_query=phenotype_query,sample_idx= sp.array(self.sample_idx["pheno"]),phenotype_IDs=phenotype_IDs,center=center,intersection=intersection)
        return self.subsample(rows=sample_idx_intersect,cols_pheno=None,cols_geno=None,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None)



import scipy.stats as st
import numpy as NP
cimport numpy as NP
from libcpp cimport bool
from numpy cimport int64_t

cdef extern from "numpy/arrayobject.h":
    ctypedef int intp
    ctypedef extern class numpy.ndarray [object PyArrayObject]:
        cdef char *data
        cdef int nd
        cdef intp *dimensions
        cdef intp *strides
        cdef int flags


cdef extern from "types.h":
    ctypedef double mfloat_t
    ctypedef int64_t mint_t


    cdef cppclass MatrixXd:
        MatrixXd()
        int rows()
        int cols()

    cdef cppclass MMatrixXd:
        MMatrixXd(mfloat_t *,int,int)
        int rows()
        int cols()

    cdef cppclass MMatrixXdRM:
        MMatrixXdRM(mfloat_t *,int,int)
        int rows()
        int cols()

    cdef cppclass MMatrixXi:
        MMatrixXi(mint_t *,int,int)
        int rows()
        int cols()


##conversion for numpy arrays
def ndarrayF64toC(ndarray A):
    #transform numpy object to enforce Fortran contiguous byte order
    #(meaining column-first order for Cpp interfacing)
    return NP.asarray(A, order="F")

cdef extern from "SplittingCore.h":
    void c_ml_beta "ml_beta" (MatrixXd* out, MMatrixXd* UTy, MMatrixXd* UTX, MMatrixXd* S, mfloat_t delta)
    #void c_ml_sigma "ml_sigma" (MatrixXd* out, MMatrixXd* UTy, MMatrixXd* UTX, MMatrixXd* S, mfloat_t delta)
    mfloat_t c_estimate_bias "estimate_bias" (
                                              MMatrixXd* UTy,
                                              MMatrixXd* UT,
                                              MMatrixXd* S,
                                              mfloat_t delta)
    void c_best_split_full_model "best_split_full_model" (
                                                          mint_t* m_best,
                                                          mfloat_t* s_best,
                                                          mfloat_t* left_mean,
                                                          mfloat_t* right_mean,
                                                          mfloat_t* ll_score,
                                                          MMatrixXdRM* X,
                                                          MMatrixXd* UTy,
                                                          MMatrixXd* C,
                                                          MMatrixXd* S,
                                                          MMatrixXd* U,
                                                          MMatrixXi* noderange,
                                                          mfloat_t delta)

    void c_predict_rec "predict_rec" (mfloat_t* response,
                              mint_t root,
                              MMatrixXi* tree_nodes,
                              MMatrixXi* left_children,
                              MMatrixXi* right_children,
                              MMatrixXi* best_predictor,
                              MMatrixXd* mean,
                              MMatrixXd* splitting_value,
                              MMatrixXd* X,
                              mfloat_t depth)

    void c_predict "predict" (MMatrixXd* response,
                              MMatrixXi* tree_nodes,
                              MMatrixXi* left_children,
                              MMatrixXi* right_children,
                              MMatrixXi* best_predictor,
                              MMatrixXd* mean,
                              MMatrixXd* splitting_value,
                              MMatrixXd* X,
                              mfloat_t depth)

    void c_test "test" (MatrixXd* out, MMatrixXd* XR,MMatrixXd* yR)
    void c_copy "copy" (MMatrixXd* m_out, MatrixXd* m_in)

def estimate_bias(ndarray UTy, ndarray UT, ndarray S, delta):
    UTy = ndarrayF64toC(UTy)
    UT = ndarrayF64toC(UT)
    S = ndarrayF64toC(S)

    cdef MMatrixXd* mUTy = new MMatrixXd(<double* > UTy.data,UTy.dimensions[0],UTy.dimensions[1])
    cdef MMatrixXd* mUT = new MMatrixXd(<double* > UT.data,UT.dimensions[0],UT.dimensions[1])
    cdef MMatrixXd* mS = new MMatrixXd(<double* > S.data,S.dimensions[0],1)
    return c_estimate_bias(mUTy, mUT, mS, delta)

def best_split_full_model(ndarray X,
                          ndarray UTy,
                          ndarray C,
                          ndarray S,
                          ndarray U,
                          ndarray noderange,
                          mfloat_t delta):

    # Note, X is also row major in Cpp
    UTy = ndarrayF64toC(UTy)
    U = ndarrayF64toC(U)
    S = ndarrayF64toC(S)
    C = ndarrayF64toC(C)

    cdef MMatrixXdRM* mX = new MMatrixXdRM(<double*> X.data,X.dimensions[0],X.dimensions[1])
    cdef MMatrixXd* mUTy = new MMatrixXd(<double*> UTy.data,UTy.dimensions[0],1)
    cdef MMatrixXd* mC = new MMatrixXd(<double*> C.data,C.dimensions[0],C.dimensions[1])
    cdef MMatrixXd* mS = new MMatrixXd(<double*> S.data,S.dimensions[0],1)
    cdef MMatrixXd* mU = new MMatrixXd(<double*> U.data,U.dimensions[0],U.dimensions[1])
    cdef MMatrixXi* mnoderange = new MMatrixXi(<mint_t* > noderange.data,noderange.dimensions[0],1)

    cdef NP.ndarray m_best=NP.zeros([1,1],dtype=NP.int64)
    cdef NP.ndarray s_best=NP.zeros([1,1],dtype=NP.float)
    cdef NP.ndarray left_mean=NP.zeros([1,1],dtype=NP.float)
    cdef NP.ndarray right_mean=NP.zeros([1,1],dtype=NP.float)
    cdef NP.ndarray ll_score=NP.zeros([1,1],dtype=NP.float)
    m_best[0,0] = -1
    s_best[0,0] = 0.0
    left_mean[0,0] = 0.0
    right_mean[0,0] = 0.0
    ll_score[0,0] = 0.0
    #References to objects passed to cpp function
    cdef mint_t* dm_best = <mint_t*> m_best.data
    cdef double* ds_best = <double*> s_best.data
    cdef double* dleft_mean = <double*> left_mean.data
    cdef double* dright_mean = <double*> right_mean.data
    cdef double* dll_score = <double*> ll_score.data

    c_best_split_full_model(
                            dm_best,
                            ds_best,
                            dleft_mean,
                            dright_mean,
                            dll_score,
                            mX,
                            mUTy,
                            mC,
                            mS,
                            mU,
                            mnoderange,
                            delta)

    return m_best[0,0], s_best[0,0], left_mean[0,0], right_mean[0,0], ll_score[0,0]

#############################################
### routines for making prediction faster ###
#############################################



def predict_rec(ndarray tree_nodes,
            ndarray left_children,
            ndarray right_children,
            ndarray best_predictor,
            ndarray mean,
            ndarray splitting_value,
            ndarray X,
            mfloat_t depth):

    X = ndarrayF64toC(X)
    cdef MMatrixXi* mtree_nodes = new MMatrixXi(<mint_t*> tree_nodes.data, tree_nodes.dimensions[0],1)
    cdef MMatrixXi* mleft_children = new MMatrixXi(<mint_t*> left_children.data, left_children.dimensions[0],1)
    cdef MMatrixXi* mright_children = new MMatrixXi(<mint_t*> right_children.data, right_children.dimensions[0],1)
    cdef MMatrixXi* mbest_predictor = new MMatrixXi(<mint_t*> best_predictor.data, best_predictor.dimensions[0],1)
    cdef MMatrixXd* mmean = new MMatrixXd(<double*> mean.data, mean.dimensions[0],1)
    cdef MMatrixXd* msplitting_value = new MMatrixXd(<double*> splitting_value.data, splitting_value.dimensions[0],1)
    cdef MMatrixXd* mX = new MMatrixXd(<double*> X.data,X.dimensions[0],X.dimensions[1])

    cdef NP.ndarray response=NP.zeros([1,1])
    cdef double* dresponse = <double*> response.data
    cdef mint_t root = 0
    c_predict_rec(dresponse, root, mtree_nodes, mleft_children, mright_children, mbest_predictor, mmean, msplitting_value, mX, depth)
    print 'passed c prediction function'
    return response[0,0]

def predict(ndarray response,
            ndarray tree_nodes,
            ndarray left_children,
            ndarray right_children,
            ndarray best_predictor,
            ndarray mean,
            ndarray splitting_value,
            ndarray X,
            mfloat_t depth):

    X = ndarrayF64toC(X)
    cdef MMatrixXi* mtree_nodes = new MMatrixXi(<mint_t*> tree_nodes.data, tree_nodes.dimensions[0],1)
    cdef MMatrixXi* mleft_children = new MMatrixXi(<mint_t*> left_children.data, left_children.dimensions[0],1)
    cdef MMatrixXi* mright_children = new MMatrixXi(<mint_t*> right_children.data, right_children.dimensions[0],1)
    cdef MMatrixXi* mbest_predictor = new MMatrixXi(<mint_t*> best_predictor.data, best_predictor.dimensions[0],1)
    cdef MMatrixXd* mmean = new MMatrixXd(<double*> mean.data, mean.dimensions[0],1)
    cdef MMatrixXd* msplitting_value = new MMatrixXd(<double*> splitting_value.data, splitting_value.dimensions[0],1)
    cdef MMatrixXd* mX = new MMatrixXd(<double*> X.data,X.dimensions[0],X.dimensions[1])

    #cdef NP.ndarray response=NP.zeros([X.dimensions[0],1])
    cdef MMatrixXd* mresponse = new MMatrixXd(<double*> response.data,response.dimensions[0],1)
    c_predict(mresponse, mtree_nodes, mleft_children, mright_children, mbest_predictor, mmean, msplitting_value, mX, depth)
    return response


###test case####
def test(ndarray Xr,ndarray yR,delta,ndarray S):
    #makre sure numpy arrays are contiguous
    Xr = ndarrayF64toC(Xr)
    yR = ndarrayF64toC(yR)
    S  = ndarrayF64toC(S)

    #map to Eigen
    cdef MMatrixXd* mXr = new MMatrixXd(<double* > Xr.data,Xr.dimensions[0],Xr.dimensions[1])
    cdef MMatrixXd* myR = new MMatrixXd(<double* > yR.data,yR.dimensions[0],yR.dimensions[1])
    cdef MMatrixXd* mS = new MMatrixXd(<double* > S.data,S.dimensions[0],S.dimensions[1])

    cdef MatrixXd* cout = new MatrixXd()
    c_test(cout,mXr,myR);
    #copy results back to python structures
    cdef NP.ndarray pout  = NP.zeros([cout.rows(),cout.cols()],dtype=NP.float,order="F")
    cdef MMatrixXd* mpout = new MMatrixXd(<double* > pout.data,cout.rows(),cout.cols())
    c_copy(mpout,cout)
    return pout



import numpy as np
from scipy.optimize import approx_fprime

_eps = np.sqrt(np.finfo(float).eps)

def mcheck_grad(func, grad, x0, epsilon=_eps,
                     allerrs=False):
    assert len(x0.shape) == 1
    errs = np.zeros(x0.size)
    x = x0.copy()
    for i in xrange(x0.size):

        def funci(x0i):
            x[i] = x0i
            r = func(x, i)
            x[i] = x0[i]
            return np.ravel(r)

        def gradi(x0i):
            x[i] = x0i
            g = np.ravel(grad(x, i))
            x[i] = x0[i]
            return g

        errs[i] = _vcheck_grad(funci, gradi, x0[i],
                                 epsilon)

    assert np.all(errs >= 0.0)
    if allerrs:
        return errs
    return np.max(errs)


########################### INTERNAL USE ONLY ###########################

def _check_grad(func, grad, x0, epsilon=_eps):
    return np.sqrt(sum((grad(x0) -
                   approx_fprime(x0, func, epsilon))**2))


def _vcheck_grad(func, grad, x0,
                   epsilon=_eps):

    assert len(func(x0).shape) == 1
    assert len(grad(x0).shape) == 1

    n = func(x0).size

    errs = np.zeros(n)

    for i in xrange(n):
        errs[i] = _check_grad(lambda x: func(x[0])[i], lambda x: grad(x[0])[i],
                             [x0], epsilon)


    return np.max(errs)



# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.

import sys
import scipy as sp
import numpy as np
import pdb
import pylab as pl
import matplotlib.pylab as plt
import scipy.stats as st
import copy
import os
import cPickle
import glob

def plot_manhattan(posCum,pv,chromBounds=None,
					thr=None,qv=None,lim=None,xticklabels=True,
					alphaNS=0.1,alphaS=0.5,colorNS='DarkBlue',colorS='Orange',plt=None,thr_plotting=None,labelS=None,labelNS=None):
	"""
	This script makes a manhattan plot
	-------------------------------------------
	posCum			cumulative position
	pv				pvalues
	chromBounds		chrom boundaries (optionally). If not supplied, everything will be plotted into a single chromosome
	qv				qvalues
					if provided, threshold for significance is set on qvalues but pvalues are plotted
	thr				threshold for significance
					default: 0.01 bonferroni correceted significance levels if qvs are not specified,
					or 0.01 on qvs if qvs specified
	lim				top limit on y-axis
					if not provided, -1.2*log(pv.min()) is taken
	xticklabels		if true, xtick labels are printed
	alphaNS			transparency of non-significant SNPs
	alphaS			transparency of significant SNPs
	plt				matplotlib.axes.AxesSubplot, the target handle for this figure (otherwise current axes)
	thr_plotting	plot only P-values that are smaller than thr_plotting to speed up plotting
    labelS           optional plotting label (significant loci)
    labelNS          optional plotting label (non significnat loci)
	"""
	if plt is None:
		plt = pl.gca()

	if thr==None:
		thr = 0.01/float(posCum.shape[0])

	if lim==None:
		lim=-1.2*sp.log10(sp.minimum(pv.min(),thr))

	if chromBounds is None:
		chromBounds = sp.array([[0,posCum.max()]])
	else:
		chromBounds = sp.concatenate([chromBounds,sp.array([posCum.max()])])


	n_chroms = chromBounds.shape[0]
	for chrom_i in range(0,n_chroms-1,2):
		pl.fill_between(posCum,0,lim,where=(posCum>chromBounds[chrom_i]) & (posCum<chromBounds[chrom_i+1]),facecolor='LightGray',linewidth=0,alpha=0.5)

	if thr_plotting is not None:
		if pv is not None:
			i_small = pv<thr_plotting
		elif qv is not None:
			i_small = qv<thr_plotting

		if qv is not None:
			qv = qv[i_small]
		if pv is not None:
			pv = pv[i_small]
		if posCum is not None:
			posCum=posCum[i_small]

	if qv==None:
		Isign = pv<thr
	else:
		Isign = qv<thr

	pl.plot(posCum[~Isign],-sp.log10(pv[~Isign]),'.',color=colorNS,ms=5,alpha=alphaNS,label=labelNS)
	pl.plot(posCum[Isign], -sp.log10(pv[Isign]), '.',color=colorS,ms=5,alpha=alphaS,label=labelS)

	if qv is not None:
		pl.plot([0,posCum.max()],[-sp.log10(thr),-sp.log10(thr)],'--',color='Gray')

	pl.ylim(0,lim)

	pl.ylabel('-log$_{10}$pv')
	pl.xlim(0,posCum.max())
	xticks = sp.array([chromBounds[i:i+2].mean() for i in range(chromBounds.shape[0]-1)])
	plt.set_xticks(xticks)
	pl.xticks(fontsize=6)

	if xticklabels:
		plt.set_xticklabels(sp.arange(1,n_chroms+1))
		pl.xlabel('genetic position')
	else:
		plt.set_xticklabels([])

	plt.spines["right"].set_visible(False)
	plt.spines["top"].set_visible(False)
	plt.xaxis.set_ticks_position('bottom')
	plt.yaxis.set_ticks_position('left')


def _qqplot_bar(M=1000000, alphaLevel = 0.05, distr = 'log10'):
	"""calculate theoretical expectations for qqplot"""
	mRange=10**(sp.arange(sp.log10(0.5),sp.log10(M-0.5)+0.1,0.1));#should be exp or 10**?
	numPts=len(mRange);
	betaalphaLevel=sp.zeros(numPts);#down in the plot
	betaOneMinusalphaLevel=sp.zeros(numPts);#up in the plot
	betaInvHalf=sp.zeros(numPts);
	for n in xrange(numPts):
	   m=mRange[n]; #numPLessThanThresh=m;
	   betaInvHalf[n]=st.beta.ppf(0.5,m,M-m);
	   betaalphaLevel[n]=st.beta.ppf(alphaLevel,m,M-m);
	   betaOneMinusalphaLevel[n]=st.beta.ppf(1-alphaLevel,m,M-m);
	betaDown=betaInvHalf-betaalphaLevel;
	betaUp=betaOneMinusalphaLevel-betaInvHalf;

	theoreticalPvals=mRange/M;
	return betaUp, betaDown, theoreticalPvals


def qqplot(pv, distr = 'log10', alphaLevel = 0.05):
	"""
	This script makes a Quantile-Quantile plot of the observed
	negative log P-value distribution against the theoretical one under the null.

	Input:
		pv				pvalues (numpy array)
		distr           scale of the distribution (log10 or chi2)
		alphaLevel      significance bounds
	"""
	shape_ok = (len(pv.shape)==1) or ((len(pv.shape)==2) and pv.shape[1]==1)
	assert shape_ok, 'qqplot requires a 1D array of p-values'

	tests = pv.shape[0]
	pnull = (0.5 + sp.arange(tests))/tests
	# pnull = np.sort(np.random.uniform(size = tests))
	Ipv = sp.argsort(pv)

	if distr == 'chi2':
	    qnull = sp.stats.chi2.isf(pnull, 1)
	    qemp = (sp.stats.chi2.isf(pv[Ipv],1))
	    xl = 'LOD scores'
	    yl = '$\chi^2$ quantiles'

	if distr == 'log10':
	    qnull = -sp.log10(pnull)
	    qemp = -sp.log10(pv[Ipv])

	    xl = '-log10(P) observed'
	    yl = '-log10(P) expected'

	plt.plot(qnull, qemp, '.')
	#plt.plot([0,qemp.m0x()], [0,qemp.max()],'r')
	plt.plot([0,qnull.max()], [0,qnull.max()],'r')
	plt.ylabel(xl)
	plt.xlabel(yl)
	if alphaLevel is not None:
	    if distr == 'log10':
	        betaUp, betaDown, theoreticalPvals = _qqplot_bar(M=tests,alphaLevel=alphaLevel,distr=distr)
	        lower = -sp.log10(theoreticalPvals-betaDown)
	        upper = -sp.log10(theoreticalPvals+betaUp)
	        plt.fill_between(-sp.log10(theoreticalPvals),lower,upper,color='grey',alpha=0.5)
	        #plt.plot(-sp.log10(theoreticalPvals),lower,'g-.')
	        #plt.plot(-sp.log10(theoreticalPvals),upper,'g-.')


def plot_normal(x=None, mean_x=None,std_x=None,color='red',linewidth=2,alpha=1,bins=20,xlim=False,plot_mean=True,plot_std=False,plot_2std=True,figure=None,annotate=True,histogram=True):
    """
    plot a fit of a normal distribution to the data in x.
    """
    import pylab
    if figure is None:
        figure=pylab.figure()
    if mean_x is None:
        #fit maximum likelihood Normal distribution mean to samples X
        mean_x = x.mean() #sample mean
    if std_x is None:
        #fit maximum likelihood Normal distribution standard deviation to samples X
        std_x = x.std()   #sample standard deviation

    xvals=np.arange(mean_x-5*std_x,mean_x+5*std_x,.001)
    yvals=st.norm.pdf(xvals,mean_x,std_x)
    #plot normal distribution:
    ax = pylab.plot(xvals,yvals,color=color,linewidth=linewidth,alpha=alpha)
    if x is not None and histogram:
        #plot histogram of x-values
        pylab.hist(x,bins,normed=True)

    if plot_mean:
        #evaluate distribution at the mean:
        max_cdf=st.norm.pdf(mean_x,mean_x,std_x)
        pylab.plot([mean_x,mean_x],[0,max_cdf],color=color,linewidth=linewidth,alpha=alpha,linestyle="--")
        if annotate:
            pylab.annotate('$\mu$', xy=(mean_x+0.6*std_x, 1.0*max_cdf),
                horizontalalignment='center', verticalalignment='center',fontsize=15,color=color)
    if plot_std:#plot mean +- 1*standard deviation (64% interval)
        std_cdf=st.norm.pdf(mean_x+std_x,mean_x,std_x)
        pylab.plot([mean_x+std_x,mean_x+std_x],[0,std_cdf],color=color,linewidth=linewidth,alpha=alpha,linestyle="--")
        pylab.plot([mean_x-std_x,mean_x-std_x],[0,std_cdf],color=color,linewidth=linewidth,alpha=alpha,linestyle="--")
        if annotate:
            pylab.annotate('$\mu+\sigma$', xy=(mean_x+1.6*std_x, 1.5*std_cdf),
                horizontalalignment='center', verticalalignment='center',fontsize=15,color=color)
    if plot_2std:#plot mean +- 2*standard deviations (95% interval)
        std2_cdf=st.norm.pdf(mean_x+2*std_x,mean_x,std_x)
        pylab.plot([mean_x+2*std_x,mean_x+2*std_x],[0,std2_cdf],color=color,linewidth=linewidth,alpha=alpha,linestyle="--")
        pylab.plot([mean_x-2*std_x,mean_x-2*std_x],[0,std2_cdf],color=color,linewidth=linewidth,alpha=alpha,linestyle="--")
        if annotate:
            pylab.annotate('$\mu+2\sigma$', xy=(mean_x+2.6*std_x, 1.5*std2_cdf),
                horizontalalignment='center', verticalalignment='center',fontsize=15,color=color)
    if xlim: #cut of unused space on y-axis
        pylab.xlim([mean_x-4*std_x,mean_x+4*std_x])
    return figure



# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pdb
import os
import numpy as SP
import csv

def deduce_delimiter(filename):
    with open(filename, 'r') as file:
        dialect = csv.Sniffer().sniff(file.read())
        return dialect.delimiter


def which(vec):
    for i in xrange(len(vec)):
        if (vec[i]):
            return(i)
    return(-1)


def readPED(basefilename, delimiter = ' ',missing = '0',standardize = True, pheno = None):
    '''
    read [basefilename].ped and [basefilename].map
    optionally standardize the SNPs and mean impute missing SNPs
    --------------------------------------------------------------------------
    Input:
    basefilename    : string of the basename of [basename].ped and [basename].map
    delimiter       : string (default ' ' space)
    missing         : string indicating a missing genotype (default '0')
    standardize     : boolean
                        True    : mean impute, zero-mean and unit variance the data
                        False   : output the data in 0,1,2 with NaN values
    --------------------------------------------------------------------------
    Output dictionary:
    'rs'     : [S] array rs-numbers, 
    'pos'    : [S*3] array of positions [chromosome, genetic dist, basepair dist],
    'snps'   : [N*S] array of snps-data,
    'iid'    : [N*2] array of family IDs and individual IDs
    --------------------------------------------------------------------------    
    '''
    pedfile = basefilename+".ped"
    mapfile = basefilename+".map"
    map = SP.loadtxt(mapfile,dtype = 'str')
    
    rs = map[:,1]
    pos = SP.array(map[:,(0,2,3)],dtype = 'float')
    map = None
    
    ped = SP.loadtxt(pedfile,dtype = 'str')
    iid = ped[:,0:2]
    snpsstr = ped[:,6::]
    inan=snpsstr==missing
    snps = SP.zeros((snpsstr.shape[0],snpsstr.shape[1]/2))
    if standardize:
        for i in xrange(snpsstr.shape[1]/2):
            snps[inan[:,2*i],i]=0
            vals=snpsstr[~inan[:,2*i],2*i:2*(i+1)]
            snps[~inan[:,2*i],i]+=(vals==vals[0,0]).sum(1)
            snps[~inan[:,2*i],i]-=snps[~inan[:,2*i],i].mean()
            snps[~inan[:,2*i],i]/=snps[~inan[:,2*i],i].std()
    else:
        for i in xrange(snpsstr.shape[1]/2):
            snps[inan[:,2*i],i]=SP.nan
            vals=snpsstr[~inan[:,2*i],2*i:2*(i+1)]
            snps[~inan[:,2*i],i]+=(vals==vals[0,0]).sum(1)
    if pheno is not None:
        #TODO: sort and filter SNPs according to pheno
        pass
    ret = {
           'rs'     :rs,
           'pos'    :pos,
           'snps'   :snps,
           'iid'    :iid
           }
    return ret

def readRAW(basefilename, delimiter = ' ',missing = '0',standardize = True, pheno = None):
    '''
    read [basefilename].raw
    optionally standardize the SNPs and mean impute missing SNPs
    --------------------------------------------------------------------------
    Input:
    basefilename    : string of the basename of [basename].ped and [basename].map
    delimiter       : string (default ' ' space)
    missing         : string indicating a missing genotype (default '0')
    standardize     : boolean
                        True    : mean impute, zero-mean and unit variance the data
                        False   : output the data in 0,1,2 with NaN values
    --------------------------------------------------------------------------
    Output dictionary:
    'rs'     : [S] array rs-numbers,
    'pos'    : [S*3] array of positions [chromosome, genetic dist, basepair dist],
    'snps'   : [N*S] array of snps-data,
    'iid'    : [N*2] array of family IDs and individual IDs
    --------------------------------------------------------------------------    
    '''
    rawfile = basefilename+".raw"
    #mapfile = basefilename+".map"
    #map = SP.loadtxt(mapfile,dtype = 'str')
    
    #rs = map[:,1]
    #pos = SP.array(map[:,(0,2,3)],dtype = 'float')
    #map = None
    import pdb
    #pdb.set_trace()
    raw = SP.loadtxt(rawfile,dtype = 'str')
    iid = raw[:,0:2]
    snpsstr = raw[:,6::]
    inan=snpsstr==missing
    snps = SP.zeros((snpsstr.shape[0],snpsstr.shape[1]/2))
    if standardize:
        for i in xrange(snpsstr.shape[1]/2):
            raw[inan[:,2*i],i]=0
            vals=snpsstr[~inan[:,2*i],2*i:2*(i+1)]
            snps[~inan[:,2*i],i]+=(vals==vals[0,0]).sum(1)
            snps[~inan[:,2*i],i]-=snps[~inan[:,2*i],i].mean()
            snps[~inan[:,2*i],i]/=snps[~inan[:,2*i],i].std()
    else:
        for i in xrange(snpsstr.shape[1]/2):
            snps[inan[:,2*i],i]=SP.nan
            vals=snpsstr[~inan[:,2*i],2*i:2*(i+1)]
            snps[~inan[:,2*i],i]+=(vals==vals[0,0]).sum(1)
    if pheno is not None:
        #TODO: sort and filter SNPs according to pheno
        pass
    ret = {
           'rs'     :rs,
           'pos'    :pos,
           'snps'   :snps,
           'iid'    :iid
           }
    return ret

def loadPhen(filename, missing ='-9', pheno = None):
    '''
    load a phenotype or covariate file. Covariates have the same file format.
    --------------------------------------------------------------------------
    Input:
    filename        : string of the filename
    missing         : string indicating a missing phenotype (default '-9')
    --------------------------------------------------------------------------
    Output dictionary:
    'header' : [P] array phenotype namesv (only if header line is specified in file),
    'vals'   : [N*P] array of phenotype-data,
    'iid'    : [N*2] array of family IDs and individual IDs
    --------------------------------------------------------------------------    
    '''
    data = SP.loadtxt(filename,dtype = 'str')
    if data[0,0] == 'FID':
        header = data[0,2::]
        data = data[1::]
    else:
        header = None
    iid = data[:,0:2]    
    data = data[:,2::]
    sel = data != missing
    sel = sel.reshape(-1)
    sel = SP.nonzero(sel)
    data = data[sel]
    iid = iid[sel,:]
    iid = iid.reshape(iid.shape[1], iid.shape[2])
    vals = SP.array(data,dtype = 'float')
    if pheno is not None:
        #TODO: sort and filter SNPs according to pheno.
        pass
    ret = {
            'header':header,
            'vals':vals,
            'iid':iid
            }
    return ret



def standardize(snps):
    N = snps.shape[0]
    S = snps.shape[1]
    for i in range(S):
        sel = snps[:,i] == snps[:,i]
        s = snps[sel,i]
        m = s.mean()
        sd = s.std()
        #print(m)
        #print(sd)
        snps[:,i] = (snps[:,i]-m)/sd
        snps[~sel,i] = 0
        #print(snps)
    return(snps)




def readBED(basefilename, blocksize = 1, start = 0, nSNPs = SP.inf, startpos = None, endpos = None, order  = 'F',standardizeSNPs=False,ipos = 2):
    '''
    read [basefilename].bed,[basefilename].bim,[basefilename].fam
    --------------------------------------------------------------------------
    Input:
    basefilename    : string of the basename of [basename].bed, [basename].bim,
                      and [basename].fam
    blocksize       : load blocksize SNPs at a time (default 1)
    start           : index of the first SNP to be loaded from the .bed-file
                      (default 0)
    nSNPs           : load nSNPs from the .bed file (default SP.inf, meaning all)
    startpos        : starting position of the loaded genomic region[chr,bpdist]
    endpos          : end-position of the loaded genomic region     [chr,bpdist]
    order           : memory layout of the returned SNP array (default 'F')
                      'F'   : Fortran-style column-major array (SNP-major)
                      'C'   : C-style row-major array (individual-major)
    standardizeSNPs : bool indeicator if the resulting SNP array is supposed to 
                      be zero-mean and unit-vatiance with mean imputed missing
                      values (default False)
    ipos            : the index of the position index to use (default 2)
                        1 : genomic distance
                        2 : base-pair distance
    --------------------------------------------------------------------------
    Output dictionary:
    'rs'     : [S] array rs-numbers
    'pos'    : [S*3] array of positions [chromosome, genetic dist, basepair dist]
    'snps'   : [N*S] array of snp-data
    'iid'    : [N*2] array of family IDs and individual IDs
    --------------------------------------------------------------------------    
    '''
    fam = basefilename+ '.fam'
    delimiter = deduce_delimiter(fam)
    fam = SP.loadtxt(fam,delimiter = delimiter,dtype = 'str',usecols=(0,1))
    bim = basefilename+'.bim'
    delimiter = deduce_delimiter(bim)
    bim = SP.loadtxt(bim,delimiter = delimiter,dtype = 'str',usecols = (0,1,2,3,4,5))
    rs = bim[:,1]
    pos = SP.array(bim[:,(0,2,3)],dtype = 'float')
    alleles = SP.array(bim[:,(4,5)],dtype='str')
    #pdb.set_trace()
    if startpos is not None:
        #pdb.set_trace()
        i_c = pos[:,0]==startpos[0]
        i_largerbp = pos[:,ipos]>=startpos[ipos]
        start = which(i_c * i_largerbp)
        while (start-1 >= 0 and pos[start-1,ipos] == startpos[ipos]):
            start = start -1
        i_c = pos[:,0]==endpos[0]
        i_smallerbp = pos[:,ipos]>=endpos[ipos]
        end = which(i_c * i_smallerbp)
        while (end+1 < pos.shape[0] and pos[end+1,ipos] == endpos[ipos]):
            end = end + 1
        nSNPs = end - start
        if (nSNPs<=0) or (end==0) or (start<=0):
            ret = {
                'pos':SP.zeros((0,3)),
                'rs':SP.zeros((0)),
                'iid':fam,
                'snps':SP.zeros((fam.shape[0],0))
                }
            return ret
        pass
    N = fam.shape[0]
    S = bim.shape[0]
    S_res = min(S,start + nSNPs)
    nSNPs = min(S-start,nSNPs)
    #if startpos is not None:
	#print("start: " + str(start))
	#print("end: " + str(end))
    #print("S_res: " + str(S_res))
    #print("nSNPs: " + str(nSNPs))
    if nSNPs<=0:
        ret = {
            'rs'     :rs[start:start],
            'pos'    :pos[start:start,:],
            #'snps'   :SNPs[0:N,start:start],
            'snps'   :SP.zeros((N,0)),
            'iid'    :fam
            }
        return ret
    SNPs = SP.zeros(((SP.ceil(0.25*N)*4),nSNPs),order=order)
    bed = basefilename + '.bed'
    with open(bed, "rb") as f:
        mode = f.read(2)
        if mode != 'l\x1b':
            raise Exception('No valid binary PED file')
        mode = f.read(1) #\x01 = SNP major \x00 = individual major
        if mode != '\x01':
            raise Exception('only SNP-major is implemented')
        startbit = SP.ceil(0.25*N)*start+3
        f.seek(startbit)
        for blockStart in SP.arange(0,nSNPs,blocksize):
            blockEnd = min(S,blockStart+blocksize)
            Sblock = min(nSNPs-blockStart,blocksize)
            nbyte = int(SP.ceil(0.25*N)*Sblock)
            bytes = SP.array(bytearray(f.read(nbyte))).reshape((SP.ceil(0.25*N),Sblock),order='F')
            
            SNPs[3::4,blockStart:blockEnd][bytes>=64]=SP.nan
            SNPs[3::4,blockStart:blockEnd][bytes>=128]=1
            SNPs[3::4,blockStart:blockEnd][bytes>=192]=2
            bytes=SP.mod(bytes,64)
            SNPs[2::4,blockStart:blockEnd][bytes>=16]=SP.nan
            SNPs[2::4,blockStart:blockEnd][bytes>=32]=1
            SNPs[2::4,blockStart:blockEnd][bytes>=48]=2
            bytes=SP.mod(bytes,16)
            SNPs[1::4,blockStart:blockEnd][bytes>=4]=SP.nan
            SNPs[1::4,blockStart:blockEnd][bytes>=8]=1
            SNPs[1::4,blockStart:blockEnd][bytes>=12]=2
            bytes=SP.mod(bytes,4)
            SNPs[0::4,blockStart:blockEnd][bytes>=1]=SP.nan
            SNPs[0::4,blockStart:blockEnd][bytes>=2]=1
            SNPs[0::4,blockStart:blockEnd][bytes>=3]=2
    
    if 0: #the binary format as described in the documentation (seems wrong)
        SNPs[3::4][bytes>=128]=SP.nan
        SNPs[3::4][bytes>=192]=1
        bytes=SP.mod(bytes,128)
        SNPs[3::4][bytes>=64]+=1
        bytes=SP.mod(bytes,64)
        SNPs[2::4][bytes>=32]=SP.nan
        SNPs[2::4][bytes>=48]=1
        bytes=SP.mod(bytes,32)
        SNPs[2::4][bytes>=16]+=1
        bytes=SP.mod(bytes,16)
        SNPs[1::4][bytes>=8]=SP.nan
        SNPs[1::4][bytes>=12]=1
        bytes=SP.mod(bytes,8)
        SNPs[1::4][bytes>=4]+=1
        bytes=SP.mod(bytes,4)
        SNPs[0::4][bytes>=2]=SP.nan
        SNPs[0::4][bytes>=3]=1
        bytes=SP.mod(bytes,2)
        SNPs[0::4][bytes>=1]+=1
    snps = SNPs[0:N,:]
    if standardizeSNPs:
        snps = standardize(snps)
    ret = {
            'rs'     :rs[start:S_res],
            'pos'    :pos[start:S_res,:],
            'alleles'    :alleles[start:S_res,:],
            'snps'   :snps,
            'iid'    :fam
            }
    return ret




def findIndex(ids, bedids): 
    N1 = ids.shape[0]
    com1 = SP.chararray(N1, itemsize=30)
    #com1 = {}
    N2 = bedids.shape[0]
    com2 = {}
    for i in range(N1):
        com1[i] = ids[i,0] + "_" + ids[i,1] 
        #com1[ ids[i,0] + "_" + ids[i,1] ] = i
    for i in range(N2):
        com2[ bedids[i,0] + "_" + bedids[i,1] ] = i
    if (N1 <= N2):
        index = SP.zeros(N1)
        count = 0
        for i in range(N1):
            try:
                ind = com2[ com1[i] ]
            except KeyError:
                continue
            index[count] = ind
            count = count + 1
    else:
        index = SP.zeros(N2)
        count = 0
        for i in range(N2):
            try:
                ind = com2[ com1[i] ]
            except KeyError:
                continue
            index[count] = ind
            count = count + 1
    index = index[0:count]
    #index = index[index != -1]
    return index.astype('I')
    
       


def filter(phe, bed):
    # for pheno
    index = findIndex(bed['iid'], phe['iid'])
    iid = phe['iid']
    iid = iid[index,:]
    vals = phe['vals']
    vals = vals[index]
    phe['iid'] = iid
    phe['vals'] = vals
    # for snp
    index = findIndex(phe['iid'], bed['iid'])
    iid = bed['iid']
    iid = iid[index,:]
    snps = bed['snps']
    snps = snps[index,:]
    bed['iid'] = iid
    bed['snps'] = snps
    return
     





#if __name__ == "__main__":
#    #datadir = "C:\\Users\\lippert\\Projects\\ARIC" ; basefilename = os.path.join(datadir, 'whiteMale')
#    datadir = "testdata" ; basefilename = os.path.join(datadir, 'test')
#    #bed = readBED(basefilename,blocksize = 1,nSNPs = 20000,start = 650000)
#    #bed = readBED(basefilename,blocksize = 1,nSNPs = 20000,start = 0)
#    #phe = loadPhen(basefilename + ".tab")
#    filter(phe, bed)
#    bed = readBED('Gaw14/all', startpos = [1,17.5599], endpos = [1,24.6047])
   


 



# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
import scipy.stats as st

def power(maf=0.5,beta=0.1, N=100, cutoff=5e-8):
	"""
	estimate power for a given allele frequency, effect size beta and sample size N

	Assumption:
	
	z-score = beta_ML distributed as p(0) = N(0,1.0(maf*(1-maf)*N))) under the null hypothesis
	the actual beta_ML is distributed as p(alt) = N( beta , 1.0/(maf*(1-maf)N) )
	

	
	Arguments:
		maf:	minor allele frequency of the SNP
		beta:	effect size of the SNP
		N:		sample size (number of individuals)
	Returns:
		power:	probability to detect a SNP in that study with the given parameters
	"""
	
	"""
	std(snp)=sqrt(2.0*maf*(1-maf)) 
	power = \int 

	beta_ML = (snp^T*snp)^{-1}*snp^T*Y = cov(snp,Y)/var(snp)   
	E[beta_ML]	= (snp^T*snp)^{-1}*snp^T*E[Y]  
				= (snp^T*snp)^{-1}*snp^T*snp * beta
				= beta
	Var[beta_ML]= (snp^T*snp)^{-1}*(snp^T*snp)*(snp^T*snp)^{-1}
				= (snp^T*snp)^{-1}
				= 1/N * var(snp)
				= 1/N * maf*(1-maf)
	"""
	assert maf>=0.0 and maf<=0.5, "maf needs to be between 0.0 and 0.5, got %f" % maf
	if beta<0.0:
		beta=-beta
	std_beta = 1.0/np.sqrt(N*(2.0 * maf*(1.0-maf)))
	non_centrality = beta
	beta_samples = np.random.normal(loc=non_centrality, scale=std_beta)
	n_grid = 100000
	beta_in = np.arange(0.5/(n_grid+1.0),(n_grid-0.5)/(n_grid+1.0),1.0/(n_grid+1.0)) 
	beta_theoretical = ((st.norm.isf(beta_in)* std_beta) + non_centrality)
	pvals = st.chi2.sf( (beta_theoretical/std_beta)*(beta_theoretical/std_beta) ,1.0) 
	
	power = (pvals<cutoff).mean()
	return power, pvals




import sys
from covar_base import Covariance
from limix.core.covar import Cov2KronSum
from limix.core.covar import LowRankCov
from limix.core.type.cached import cached
from limix.core.type.exception import TooExpensiveOperationError
from limix.core.utils import my_name
from util import msg_too_expensive_dim
import scipy as sp
import numpy as np
import scipy.linalg as la
import warnings

import pdb

_MAX_DIM = 5000

class Cov3KronSumLR(Cov2KronSum):
    """
    Covariance class for sum of 3 Kronecker products with one low rank term:
        K = Cr \kron GG.T + Cg \kron R + Cn \kron I
    Notation:
        - dim_c: dimension of col covariances
        - dim_r: dimension of row covariances
        - rank_c: rank of low-rank col covariance
        - rank_r: rank of low-rank row covariance
    """

    def __init__(self, Cg=None, Cn=None, G=None, R=None, rank=1, Cr=None, S_R=None, U_R=None):
        """
        Args:
            Cg:     Limix covariance matrix for Cg (dimension dim_c)
            Cn:     Limix covariance matrix for Cn (dimension dim_c)
            G:      [dim_r, rank_r] numpy covariance matrix for G
            R:      [dim_r, dim_r] numpy semidemidefinite covariance matrix for R
            rank:   rank of column low-rank covariance (default = 1)
            Cr:     Limix covariance matrix for Cr (optional).
                    If not specified, a low-rank covariance matrix is considered
            S_R:    N vector of eigenvalues of R
            U_R:    [N, N] eigenvector matrix of R
        """
        Covariance.__init__(self)
        self._Cr_act = True
        self._Cg_act = True
        self._Cn_act = True
        self.setColCovars(Cg=Cg, Cn=Cn, rank=rank, Cr=None)
        self.setR(R=R, S_R=S_R, U_R=U_R)
        self.G = G
        self.dim = self.dim_c * self.dim_r
        self._calcNumberParams()
        self._use_to_predict = False

    def G_has_changed(self):
        self.clear_cache('G')
        self._notify('G')
        self.clear_all()

    #####################
    # Properties
    #####################
    @property
    def rank_r(self):
        return self._rank_r

    @property
    def rank_c(self):
        return self._rank_c

    @property
    def Cr(self):
        return self._Cr

    @property
    def G(self):
        return self._G

    #####################
    # Setters
    #####################
    @G.setter
    def G(self, value):
        assert value is not None, '%s: Specify G!' % self.__class__.__name__
        assert self.dim_r == value.shape[0], '%s'
        self._G = value
        self._rank_r = value.shape[1]
        self.G_has_changed()

    def setG(self, value):
        self.G = value

    # normal setter for col covars
    def setColCovars(self, Cg=None, Cn=None, rank=1, Cr=None):
        assert Cg is not None, 'Cov2KronSum: Specify Cg!'
        assert Cn is not None, 'Cov2KronSum: Specify Cn!'
        assert Cg.dim==Cn.dim, 'Cov2KronSum: Cg and Cn must have same dimensions!'
        assert Cr is None, '%s: more general covariance matrices are not supported at the moment' % self.__class__.__name__
        if Cr is None:
            Cr = LowRankCov(Cg.dim, rank)
            Cr.setRandomParams()
            self._rank_c = rank
        else:
            self._rank_c = self.Cr.dim()
        self._dim_c = Cg.dim
        self._Cr = Cr
        self._Cg = Cg
        self._Cn = Cn
        self._Cg.register(self.col_covs_have_changed)
        self._Cn.register(self.col_covs_have_changed)
        self.col_covs_have_changed()

    #####################
    # Activation handling
    #####################
    @property
    def act_Cr(self):
        return self._Cr_act

    @act_Cr.setter
    def act_Cr(self, act):
        self._Cr_act = bool(act)
        self._notify()

    @property
    def act_Cg(self):
        return self._Cg_act

    @act_Cg.setter
    def act_Cg(self, act):
        self._Cg_act = bool(act)
        self._notify()

    @property
    def act_Cn(self):
        return self._Cn_act

    @act_Cn.setter
    def act_Cn(self, act):
        self._Cn_act = bool(act)
        self._notify()

    def _actindex2index(self, i):
        nCr = self.Cr.getNumberParams()
        nCg = self.Cg.getNumberParams()
        i += nCr * int(not self._Cr_act)
        i += nCg * int(not self._Cg_act)
        return i

    def _index2actindex(self, i):
        nCr = self.Cr.getNumberParams()
        nCg = self.Cg.getNumberParams()
        i -= nCr * int(not self._Cr_act)
        i -= nCg * int(not self._Cg_act)
        return i

    #####################
    # Params handling
    #####################
    def setParams(self,params):
        np_r = self.Cr.getNumberParams()
        np_g = self.Cg.getNumberParams()
        np_n = self.Cn.getNumberParams()

        if len(params) != self.getNumberParams():
            raise ValueError("The number of parameters passed to setParams "
                             "differs from the number of active parameters.")

        self.Cr.setParams(params[:np_r])
        self.Cg.setParams(params[np_r:(np_r + np_g)])
        self.Cn.setParams(params[(np_r + np_g):])

    def getParams(self):
        params = []
        if self._Cr_act:
            params.append(self.Cr.getParams())
        if self._Cg_act:
            params.append(self.Cg.getParams())
        if self._Cn_act:
            params.append(self.Cn.getParams())
        if len(params) == 0:
            return np.array([])
        return sp.concatenate(params)

    def getNumberParams(self):
        np_r = self.Cr.getNumberParams()
        np_g = self.Cg.getNumberParams()
        np_n = self.Cn.getNumberParams()
        nact = (np_r * int(self._Cr_act) + np_g * int(self._Cg_act) +
                np_n * int(self._Cn_act))
        return nact

    def _calcNumberParams(self):
        self.n_params = self.Cr.getNumberParams() + self.Cg.getNumberParams() + self.Cn.getNumberParams()


    #####################
    # Cached
    #####################
    @cached('G')
    def GG(self):
        if self.dim > _MAX_DIM:
            raise TooExpensiveOperationError(msg_too_expensive_dim(my_name(),
                                                                   _MAX_DIM))

        return sp.dot(self.G, self.G.T)

    @cached(['G', 'row_cov'])
    def Wr(self):
        return sp.dot(self.Lr(), self.G)

    @cached('col_cov')
    def Wc(self):
        E = self.Cr.X
        return sp.dot(self.Lc(), E)

    # no need to cached
    def W(self):
        return sp.kron(self.Wc(), self.Wr())

    @cached(['col_cov', 'row_cov', 'G'])
    def DW(self):
        return self.d()[:, sp.newaxis] * self.W()

    @cached(['col_cov', 'row_cov', 'G'])
    def DWt(self):
        return self.DW().reshape((self._dim_r, self.dim_c, self.rank_r * self.rank_c), order = 'F')

    @cached(['col_cov', 'row_cov', 'G'])
    def H_chol(self):
        H = np.tensordot(self.DWt(),self.Wc(),axes=(1,0))
        H = np.transpose(H, (0, 2, 1))
        H = np.tensordot(self.Wr(), H, axes=(0,0))
        H = H.reshape((self.rank_c * self.rank_r, self.rank_c * self.rank_r), order='F')
        H+= np.eye(self.rank_r * self.rank_c)
        return la.cholesky(H).T

    @cached(['col_cov', 'row_cov', 'G'])
    def H_inv(self):
        return la.cho_solve((self.H_chol(), True), sp.eye(self.rank_r * self.rank_c))

    @cached('col_cov')
    def LcGradCrLc(self, i):
        return sp.dot(self.Lc(), sp.dot(self.Cr.K_grad_i(i), self.Lc().T))

    @cached('col_cov')
    def Ctilde(self, i):
        np_r = self.Cr.getNumberParams()
        np_g = self.Cg.getNumberParams()
        if i < np_r:
            r = self.LcGradCrLc(i)
        elif i < (np_r + np_g):
            _i = i - np_r
            r = self.LcGradCgLc(_i)
        else:
            _i = i - np_r - np_g
            r = self.LcGradCnLc(_i)
        return r

    @cached(['row_cov', 'G'])
    def diagWrWr(self):
        return (self.Wr()**2).sum(1)

    @cached(['col_cov', 'row_cov', 'G'])
    def diag_Ctilde_o_Sr(self, i):
        np_r = self.Cr.getNumberParams()
        np_g = self.Cg.getNumberParams()
        if i < np_r:
            r = sp.kron(sp.diag(self.LcGradCrLc(i)), self.diagWrWr())
        elif i < (np_r + np_g):
            _i = i - np_r
            r = sp.kron(sp.diag(self.LcGradCgLc(_i)), self.Sr())
        else:
            _i = i - np_r - np_g
            r = sp.kron(sp.diag(self.LcGradCnLc(_i)), sp.ones(self.dim_r))
        return r

    @cached(['col_cov', 'row_cov', 'G'])
    def WrWrDWt(self):
        R = np.tensordot(self.Wr(), self.DWt(), axes=(0,0))
        R = np.tensordot(self.Wr(), R, axes=(1,0))
        return R

    @cached(['col_cov', 'row_cov', 'G'])
    def SrDWt(self):
        return self.Sr()[:, sp.newaxis, sp.newaxis] * self.DWt()

    @cached(['col_cov', 'row_cov', 'G'])
    def Kbar(self, i):
        # WrDRDWtCtildeWc
        np_r = self.Cr.getNumberParams()
        np_g = self.Cg.getNumberParams()
        if i < np_r:
            RDWt = self.WrWrDWt()
        elif i < (np_r + np_g):
            RDWt = self.SrDWt()
        else:
            RDWt = self.DWt()
        ping = np.tensordot(RDWt, self.Ctilde(i), axes=(1,0))
        pong = self.D()[:, sp.newaxis, :] * ping
        ping = np.tensordot(pong, self.Wc(), axes=(2,0))
        pong = np.tensordot(self.Wr(), ping, axes=(0,0))
        ping = np.transpose(pong, (0,2,1))
        return ping.reshape((self.rank_c * self.rank_r, self.rank_c * self.rank_r),order='F')

    def Sr_X_Ctilde(X, i):
        pass

    #####################
    # Overwritten covar_base methods
    #####################
    @cached(['col_cov', 'row_cov', 'G', 'covar_base'])
    def K(self):
        if self.dim > _MAX_DIM:
            raise TooExpensiveOperationError(msg_too_expensive_dim(my_name(),
                                                                   _MAX_DIM))

        R  = sp.kron(self.Cr.K(), self.GG())
        R += sp.kron(self.Cg.K(), self.R)
        R += sp.kron(self.Cn.K(), sp.eye(self.dim_r))
        return R

    @cached(['col_cov', 'row_cov', 'G', 'covar_base'])
    def K_grad_i(self,i):
        np_r = self.Cr.getNumberParams()
        np_g = self.Cg.getNumberParams()
        np_n = self.Cn.getNumberParams()

        if i >= self.getNumberParams():
            raise ValueError("Trying to retrieve the gradient over a "
                             "parameter that is inactive.")

        if self.dim > _MAX_DIM:
            raise TooExpensiveOperationError(msg_too_expensive_dim(my_name(),
                                                                   _MAX_DIM))

        i = self._actindex2index(i)

        if i < np_r:
            rv= sp.kron(self.Cr.K_grad_i(i), self.GG())
        elif i < (np_r + np_g):
            _i = i - np_r
            rv = sp.kron(self.Cg.K_grad_i(_i), self.R)
        else:
            _i = i - np_r - np_g
            rv = sp.kron(self.Cn.K_grad_i(_i), sp.eye(self.dim_r))
        return rv

    @cached(['col_cov', 'row_cov', 'G', 'covar_base'])
    def logdet(self):
        r = sp.log(self.SpI()).sum()
        r+= sp.sum(sp.log(self.Cn.S())) * self.dim_r
        r+= 2 * sp.log(sp.diag(self.H_chol())).sum()
        return r

    @cached(['col_cov', 'row_cov', 'G', 'covar_base'])
    def logdet_grad_i(self, i):
        r = (self.d() * self.diag_Ctilde_o_Sr(i)).sum()
        r-= (self.H_inv() * self.Kbar(i)).sum()
        return r


if __name__ == '__main__':
    from limix.core.covar import FreeFormCov
    from limix.utils.preprocess import covar_rescale

    # define row caoriance
    dim_r = 10
    X = sp.rand(dim_r, dim_r)
    R = covar_rescale(sp.dot(X,X.T))

    # define col covariances
    dim_c = 3
    Cg = FreeFormCov(dim_c)
    Cn = FreeFormCov(dim_c)

    # cov = Cov3KronSum(Cg = Cg, Cn = Cn, R = R)
    # cov.setRandomParams()
    #
    # print cov.K()
    # print cov.K_grad_i(0)



'''
Created on Apr 2, 2013

@author: johannes
'''

import scipy as SP
import scipy.linalg as LA
import warnings
import py_splitting_core as SC
import par_lmm_forest
import lmm_forest_utils as utils
import blup as BLUP
from limix.ensemble import SplittingCore as CSP

class Forest(object):
    '''
    classdocs
    '''
    def __init__(self,
                 n_estimators=100,
                 max_depth=float('inf'),
                 min_samples_split=5,
                 ratio_features=.66,
                 compute_importances=False,
                 fit_optimal_depth=False,
                 build_to_opt_depth=False,
                 pruning='err',
                 min_depth=0,
                 update_delta=False,
                 optimize_memory_use=True,
                 oob_score=False,
                 tc=None,
                 random_state=None,
                 kernel='data',
                 delta=None,
                 subsampling=True,
                 sampsize=.5,
                 verbose=0,
                 max_nodes=float('inf'),
                 cpp_fit=True,
                 cpp_predict=True):
        '''
        A linear mixed forest.
        A linear mixed forest is a meta estimator derived from the random forest
        that fits a number of linear mixed decision trees on various sub-samples
        of the data set and uses averaging
        to improve the predictive accuracy and control over-fitting.

        Parameters
        ----------

        kernel: accepts either
            (1) positive definite matrix of size n x n, describing a dependency
                structure between the n samples (e.g. population structure).
            (2) strings 'data' or 'iid'
            If set to 'data' dependency structure is estimated from the
            predictor matrix X, if set to 'iid' i.i.d. samples are assumed
            and the linear mixed forest behaves like the random forest.

        n_estimators : integer, optional (default=100)
            The number of trees in the forest.

        ratio_features : fraction of the total number of features used for each
                         split (default=0.66)

        max_depth : optional (default='inf') real value that defining the
                    maximum depth of the tree.

        min_samples_split : integer, optional (default=5)
            The minimum number of samples required to split a node.

        fit_opt_depth: uses the forest likelihood to fit the optimal depth of
                       the trees in the ensemble on the out of bag samples
                       (default=False)

        build_to_opt_depth: stops furthering trees in the ensemble if
                            improvement of forest likelihood stops
                            (default=False). This helps to improve runtime
                            and to avoid overfitting when a relatively small
                            number of trees is used on a big dataset.

        sampsize: The number of samples drawn without replacement for growing a
                  single tree (default=0.5)

        subsampling: If 'True' this toggles sampling without replacement.
                     Otherwise as sampling without replacement is used
                     (default=True)

        verbose : int, optional (default=0) Controls the verbosity level of the
                  tree building process.
        '''
        self.init_parameters = locals()
        self.init_parameters.pop('self')
        self.n_estimators = n_estimators
        self.max_depth = max_depth
        self.min_samples_split = min_samples_split
        self.random_state = random_state
        self.kernel = kernel
        self.delta = delta
        self.update_delta = update_delta
        self.compute_importances = compute_importances
        self.fit_optimal_depth = fit_optimal_depth
        self.build_to_opt_depth = build_to_opt_depth
        self.optimize_memory_use = optimize_memory_use
        self.oob_error = None
        self.var_used = None
        self.log_importance = None
        self.ratio_features = ratio_features
        self.subsampling = subsampling
        self.sampsize = sampsize
        self.verbose = verbose
        self.depth = 0
        self.opt_depth = None
        self.pruning = pruning
        self.min_oob_error = float('inf')
        self.opt_delta = None
        self.fixed_effect = None
        self.tc = tc
        self.max_nodes = max_nodes
        self.cpp_fit = cpp_fit
        self.cpp_predict = cpp_predict
        self.min_depth = min_depth

        # Settings for (re)initialization#
        self.X = None
        self.y = None
        self.n = None
        self.m = None
        self.BLUP = None
        self.max_features = None
        self.var_used = None
        self.log_importance = None
        self.trees = []

        if random_state is not None:
            SP.random.seed(random_state)
        if tc is not None:
            parMixedForest.par_init(**self.init_parameters)

    def reset(self):
        # Reinstatiate using the same parameters
        self.__init__(**self.init_parameters)

    def get_params(self, deep=True):
        return self.init_parameters

    # TODO: this is quite a hack for making scikits crossvalidation work
    def set_params(self, params):
        self.__init__(**params)

    def fit(self, X, y, recycle=True, **grow_params):

        """Build a linear mixed forest of trees from the training set (X, y).

        Parameters
        ----------
        X : array-like of shape = [n_samples, n_features]
            The training input samples.

        y : array-like, shape = [n_samples] or [n_samples, 1]
            The real valued targets

        Returns
        -------
        self : object
            Returns self.
        """
        if isinstance(self.kernel, str) and self.kernel == 'data':
            self.kernel = SC.estimateKernel(X, maf=1.0/X.shape[0])
        elif isinstance(self.kernel, str) and self.kernel == 'iid':
            self.kernel = SP.identity(X.shape[0])
        # Use dedicated part of data as background model
        elif self.kernel.size == X.shape[1]:
            tmp_ind = self.kernel
            self.kernel = utils.estimateKernel(X[:, self.kernel],
                                               maf=1.0/X.shape[0])
            X = X[:, ~tmp_ind]
        # Extract and reshape data
        self.y = y.reshape(-1, 1)
        self.X = X
        self.n, self.m = self.X.shape
        if self.delta is None:
            self.BLUP = BLUP.BLUP()
            if self.verbose > 1:
                print 'fitting BLUP'
            self.BLUP.fit(XTrain=self.X, yTrain=self.y, KTrain=self.kernel,
                          delta=self.delta)
            if self.verbose > 1:
                print 'done fitting BLUP'
            # Update delta if it used to be 'None'
            self.delta = self.BLUP.delta
        self.max_features = SP.maximum(SP.int_(self.ratio_features*self.m), 1)
        self.var_used = SP.zeros(self.m)
        self.log_importance = SP.zeros(self.m)
        self.depth = 0

        if self.verbose > 0:
            print 'log(delta) fitted to ', SP.log(self.delta)

        # Initialize individual trees
        if recycle and self.trees != []:
            for tree in self.trees:
                tree.cut_to_stump()
        else:
            n_trees = 0
            self.trees = []
            while n_trees < self.n_estimators:
                if self.verbose > 1:
                    print 'init. tree number ', n_trees
                subsample = self.tree_sample()
                tree = MixedForestTree(self, subsample)
                self.trees.append(tree)
                n_trees += 1

        # Fitting with optimal depth constraint
        if self.fit_optimal_depth or self.update_delta:
            self.opt_depth = 0
            self.min_oob_err = self.get_oob_error(self.depth)
            if self.verbose > 0:
                print 'initial oob error is:', self.min_oob_err
            grow_further = True
            curr_depth = self.depth
            while grow_further:
                # Updating ensemble increasing its depth by one
                self.further(depth=self.depth+1)
                if self.update_delta:
                    self.delta = self.delta_update()
                    if self.verbose > 0:
                        print 'delta was fitted to', self.delta
                if self.verbose > 0:
                    print 'depth is:', self.depth
                oob_err = self.get_oob_error(self.depth)
                if self.verbose > 0:
                    print 'oob error is:', oob_err
                if oob_err < self.min_oob_err:
                    self.min_oob_err = oob_err
                    self.opt_depth = self.depth
                # Decide whether tree needs to be furthered
                grow_further = (curr_depth < self.depth) and\
                    (self.depth < self.max_depth)
                if self.build_to_opt_depth and (self.depth >= self.min_depth):
                    grow_further = grow_further and\
                        (oob_err == self.min_oob_err)
                    pass
                curr_depth = self.depth
        #####################################################
        # Growing full tree one by one
        else:
            self.further(depth=self.max_depth)
        return self

    def clear(self):
        if self.tc is not None:
            parMixedForest.clearMemory(self.tc)

    def tree_sample(self):
        if self.subsampling:
            n_sample = SP.int_(self.n*self.sampsize)
            subsample = SP.random.permutation(self.n)[:n_sample]
        else:
            subsample = SP.random.random_integers(0, self.n-1, self.n)
        return subsample

    def further(self, depth=float('inf')):
        if self.tc is None:
            for i in SP.arange(len(self.trees)):
                if self.verbose > 1:
                    print 'growing tree ', i
                self.trees[i].grow(depth)
                if depth == float('inf'):
                    self.trees[i].clear_data()

        else:
            self.depth = parMixedForest.par_further(self.tc, depth)

    def _predict(self, X=None, depth=float('inf'), oob=False, conf=False):
        if X is None:
            n_res = self.n
        else:
            n_res = X.shape[0]
        response = SP.zeros(n_res)
        count = SP.zeros(n_res)
        for tree in self.trees:
            if oob:
                response[tree.oob] += tree.predict(depth=depth, oob=oob,
                                                   conf=conf)
                count[tree.oob] += 1
            elif X is None:
                response[tree.subsample] += tree.predict(depth=depth, oob=oob,
                                                         conf=conf)
                count[tree.subsample] += 1
            else:
                response += tree.predict(X=X, depth=depth, conf=conf)
                count += 1
        response[count != 0] /= count[count != 0]
        if (count == 0).sum() > 1:
            warnings.warn('not all samples have been used for learning\
                          please use more trees')
            response[count == 0] = SP.mean(response)
        return response

    def get_oob_error(self, depth):
        return ((self._predict(depth=depth, conf=True, oob=True) -
                 self.y.reshape(-1))**2).sum()/self.n

    def get_oob_likelihood(self, depth):
        oob_fixed_effect = self._predict(depth=depth, oob=True).reshape(-1, 1)
        return self.BLUP.LL(oob_fixed_effect)

    def get_training_likelihood(self, depth):
        oob_fixed_effect = self._predict(depth=depth, oob=False).reshape(-1, 1)
        return self.BLUP.LL(oob_fixed_effect)

    def delta_update(self):
        mean = self._predict(depth=self.depth, oob=False)
        self.BLUP.update_delta(mean)
        return self.BLUP.delta

    def predict(self, X, k=None, depth=None):
        """Predict response for X.

        The response to an input sample is computed as the sum of
        (1) the mean prediction of the trees in the forest (fixed effect) and
        (2) the estimated random effect.

        Parameters
        ----------
        X : array-like of shape = [n_samples, n_features]
            The input samples.
        k: array-like of shape = [n_samples, n_samples_fitting]
            The cross-dependency structure between the samples used for learning
            the forest and the input samples.
            If not specified only the estimated fixed effect is returned.

        Returns
        -------
        y : array of shape = [n_samples, 1]
            The response
        """
        if depth is None:
            if self.opt_depth is not None:
                depth = self.opt_depth
                if self.verbose > 0:
                    'using optimal depth to predict'
            else:
                depth = float('inf')
        response = self._predict(X=X, depth=depth)
        if k is not None:
                mean = self.predict(X=self.X, depth=depth).reshape(-1, 1)
                response += self.BLUP.predict(XTest=X, k=k,
                                              mean=mean).reshape(-1)
        return response


class MixedForestTree(object):
    '''
    classdocs
    '''
    def __init__(self, forest, subsample):
        '''
        Constructor
        '''
        self.forest = forest
        if self.forest is not None:
            self.verbose = self.forest.verbose
        else:
            self.verbose = 0
        self.max_depth = 0
        # Estimate the potential number of nodes
        self.subsample = subsample
        self.subsample_bin = SP.zeros(self.forest.n, dtype='bool')
        self.subsample_bin[self.subsample] = True
        self.oob = SP.arange(self.forest.n)[~self.subsample_bin]
        nr_nodes = 4*subsample.size
        self.nodes = SP.zeros(nr_nodes, dtype='int')
        self.best_predictor = SP.empty(nr_nodes, dtype='int')
        self.start_index = SP.empty(nr_nodes, dtype='int')
        self.end_index = SP.empty(nr_nodes, dtype='int')
        self.left_child = SP.zeros(nr_nodes, dtype='int')
        self.right_child = SP.zeros(nr_nodes, dtype='int')
        self.parent = SP.empty(nr_nodes, dtype='int')
        self.mean = SP.zeros(nr_nodes)
        # Initialize root node
        self.node_ind = 0
        self.nodes[self.node_ind] = 0
        self.start_index[self.node_ind] = 0
        self.end_index[self.node_ind] = subsample.size
        self.num_nodes = 1
        self.num_leafs = 0
        self.s = SP.ones_like(self.nodes)*float('inf')
        kernel = self.get_kernel()
        if not(self.forest.optimize_memory_use):
            self.X = self.forest.X[subsample]
        if self.verbose > 1:
            print 'compute tree wise singular value decomposition'
        self.S, self.U = LA.eigh(kernel + SP.eye(subsample.size)*1e-8)
        self.Uy = SP.dot(self.U.T, self.forest.y[subsample])
        if self.verbose > 1:
            print 'compute tree wise bias'
        self.mean[0] = SC.estimate_bias(self.Uy, self.U, self.S,
                                        SP.log(self.forest.delta))
        self.sample = SP.arange(subsample.size)
        ck = self.get_cross_kernel(self.oob, self.subsample)
        self.cross_core = SP.dot(ck, LA.inv(kernel +
                                            SP.eye(self.subsample.size) *
                                            self.forest.delta))
        if self.verbose > 1:
            print 'done initializing tree'


    def print_tree(self):
        return self.print_tree_rec(0)

    def print_tree_rec(self, node_ind):
        if SC.is_leaf(node_ind, self.left_child):
            return SP.str_(self.mean[node_ind])
        else:
            left_child = self.left_child[node_ind]
            right_child = self.right_child[node_ind]
            return '(' + self.print_tree_rec(left_child) + ','\
                + self.print_tree_rec(right_child) + ')'

    def grow(self, depth=float('inf')):
        while (self.node_ind < self.num_nodes) and\
              (self.get_depth(self.nodes[self.node_ind]) < depth) and\
              (self.num_leafs <= self.forest.max_nodes-2):
            if (self.end_index[self.node_ind] -
               self.start_index[self.node_ind]) > self.forest.min_samples_split:

                best_j, best_s, left_mean, right_mean, ll_score =\
                    self.split_node(self.node_ind)

                if best_j != -1:
                    self.best_predictor[self.node_ind] = best_j
                    self.s[self.node_ind] = best_s
                    self.forest.var_used[best_j] += 1
                    self.forest.log_importance[best_j] += ll_score
                    split = self.rearrange(best_j, best_s, self.node_ind)
                    self.nodes[self.num_nodes] = self.nodes[self.node_ind]*2 + 1
                    self.nodes[self.num_nodes+1] =\
                        self.nodes[self.node_ind]*2 + 2
                    self.mean[self.num_nodes] = left_mean
                    self.mean[self.num_nodes+1] = right_mean
                    self.start_index[self.num_nodes] =\
                        self.start_index[self.node_ind]
                    self.start_index[self.num_nodes+1] = split
                    self.end_index[self.num_nodes] = split
                    self.end_index[self.num_nodes+1] =\
                        self.end_index[self.node_ind]
                    self.left_child[self.node_ind] = self.num_nodes
                    self.right_child[self.node_ind] = self.num_nodes + 1
                    self.parent[self.num_nodes] = self.node_ind
                    self.parent[self.num_nodes+1] = self.node_ind
                    self.forest.depth =\
                        SP.maximum(self.get_depth(self.nodes[self.num_nodes]),
                                   self.forest.depth)
                    if self.forest.depth > 50:
                        warnings.warn('tree depth exceeded 50, check for\
                                      imbalanced predictors and consider to\
                                      restrict the maximum depth of trees')
                    self.num_nodes += 2
                    self.num_leafs += 1
            self.node_ind += 1

    def get_depth(self, node):
            depth = SP.log2(node+1)
            return SP.floor(depth)

    def get_kernel(self):
        return self.forest.kernel[SP.ix_(self.subsample, self.subsample)]

    def get_cross_kernel(self, oob, subsample):
        return (self.forest.kernel +
                self.forest.delta*SP.eye(self.forest.n))[SP.ix_(oob, subsample)]

    def predict(self, X=None, depth=float('inf'), oob=False, conf=False):
        ##################################################
        # getting correct X
        if X is None:
            if oob:
                X = self.forest.X[self.oob]
            else:
                X = self.get_X()
        ##################################################
        # making prediction
        if self.forest.cpp_predict:
            response = self.predict_cpp(X, depth)
        else:
            response = self.predict_py(X, depth)
        if conf:
            mean = self.predict(self.forest.X[self.subsample], depth=depth)
            # TODO this dot product can be cached
            response += SP.dot(self.cross_core,
                               self.forest.y[self.subsample].reshape(-1)-mean)
        return response

    def predict_cpp(self, X, depth):
        #ensure consistency to c data type
        X = SP.array(X, dtype='float')
        response = SP.empty(X.shape[0])
        return CSP.predict(response, self.nodes, self.left_child,
                           self.right_child, self.best_predictor, self.mean,
                           self.s, X, depth)

    def predict_py(self, X, depth):
        response = SP.empty(X.shape[0])
        for i in SP.arange(X.shape[0]):
            response[i] = self.predict_rec(0, X[i, :], depth)
        return response

    def predict_rec(self, node_ind, x, depth):
        if self.get_depth(self.nodes[node_ind]) == depth or SC.is_leaf(node_ind,
                          self.left_child):
            return self.mean[node_ind]
        else:
            if x[self.best_predictor[node_ind]] < self.s[node_ind]:
                # Go to left child node
                return self.predict_rec(self.left_child[node_ind], x, depth)
            else:
                # Else go to right child node
                return self.predict_rec(self.right_child[node_ind], x, depth)

    def rearrange(self, j, best_s, node_ind):
        node_start = self.start_index[node_ind]
        node_end = self.end_index[node_ind]
        nodesamples = self.sample[node_start:node_end]
        left_indexes = self.get_X_slice(j)[nodesamples] < best_s
        # Print left_indexes
        left_samples = nodesamples[left_indexes]
        right_samples = nodesamples[~left_indexes]
        left_end = node_start+left_indexes.sum()
        # Resort to make split possible
        self.sample[node_start:node_end] =\
            SP.hstack((left_samples, right_samples))
        return left_end

    def split_node(self, node_ind):
        noderange = self.sample[self.start_index[node_ind]:self.end_index[node_ind]]
        rmind = SP.random.permutation(self.forest.m)[:self.forest.max_features]
        mBest = -1
        sBest = -float('inf')
        left_mean = None
        right_mean = None
        ll_score = -float('inf')
        if rmind.size > 10 and self.forest.optimize_memory_use:
            n_slice = 10
        else:
            n_slice = 1
        intv = SP.floor(n_slice*SP.arange(rmind.size)/rmind.size)
        for slc in SP.arange(n_slice):
            rmind_slice = rmind[intv==slc]
            X = self.get_X_slice(rmind_slice)
            Covariates = SC.get_covariates(node_ind,
                                self.nodes[node_ind],
                                self.parent,
                                self.sample,
                                self.start_index,
                                self.end_index)

            if self.forest.cpp_fit:
                    mBest_, sBest_, left_mean_, right_mean_, ll_score_ =\
                        SC.cpp_best_split_full_model(X, self.Uy, Covariates,
                                                     self.S,
                                                     self.U, noderange,
                                                     self.forest.delta)
            else:
                mBest_, sBest_, left_mean_, right_mean_, ll_score_ =\
                    SC.best_split_full_model(X, self.Uy, Covariates, self.S,
                                             self.U, noderange,
                                             self.forest.delta)

            if (mBest_ != -1) and (ll_score_ > ll_score):
                mBest = rmind_slice[mBest_]
                sBest = sBest_
                left_mean = left_mean_
                right_mean = right_mean_
                ll_score = ll_score_
        return mBest, sBest, left_mean, right_mean, ll_score

    def cut_to_stump(self):
        self.max_depth = 0
        self.node_ind = 0
        self.nodes[self.node_ind] = 0
        self.start_index[self.node_ind] = 0
        self.end_index[self.node_ind] = self.subsample.size
        self.num_nodes = 1
        self.num_leafs = 0
        self.left_child = SP.zeros_like(self.left_child)
        self.right_child = SP.zeros_like(self.right_child)

    def clear_data(self):
        '''
        Free memory
        If many trees are grown this is an useful options since it is saving a
        lot of memory '''
        if self.forest.verbose > 1:
            print 'clearing up stuff'
        self.S = None
        self.Uy = None
        self.U = None

    def get_X(self):
        if self.forest.optimize_memory_use:
            return self.forest.X[self.subsample]
        else:
            return self.X

    def get_X_slice(self, rmind):
        '''get X with slicing'''
        if self.forest.optimize_memory_use:
            return self.forest.X[:, rmind][self.subsample]
        else:
            return self.X[:, rmind]



import numpy as np
import scipy as sp
import scipy.linalg as la
from limix.core.type.cached import cached
from covar_base import Covariance
import pdb

class LowRankCov(Covariance):
    """
    Low-rank semi-definite positive matrix.
    A low-rank semi-definite matrix has dim * rank parameters where:
        dim = dimension of the low-rank covariance
        rank = rank of the low-rank covariance
    """
    def __init__(self, dim, rank = 1):
        """
        Args:
            dim:        dimension of the low-rank covariance
            rank:       rank of the low-rank covariance
        """
        Covariance.__init__(self)
        self.initialize(dim, rank)

    def initialize(self, dim, rank):
        self._X_act = True
        self.dim = dim
        self.rank = rank
        self._calcNumberParams()
        self.params = np.zeros(self.n_params)
        self._use_to_predict = False

    #####################
    # Properties
    #####################
    @property
    def X(self):
        return self.params.reshape((self.dim, self.rank), order='F')

    @property
    def X_ste(self):
        print 'Implement me'

    @X.setter
    def X(self, value):
        assert self.X.shape[0]==self.dim, 'Dimension mismatch.'
        assert self.X.shape[1]==self.rank, 'Dimension mismatch.'
        self.setParams(self.X.reshape(self.X.size, order = 'F'))

    #####################
    # Activation handling
    #####################
    @property
    def act_X(self):
        return self._X_act

    @act_X.setter
    def act_X(self, act):
        self._X_act = bool(act)
        self._notify()

    #####################
    # Params handling
    #####################
    def setParams(self, params):
        if not self._X_act and len(params) > 0:
            raise ValueError("Trying to set a parameter via setParams that "
                             "is not active.")
        if self._X_act:
            self.params[:] = params
            self.clear_all()

    def getParams(self):
        if not self._X_act:
            return np.array([])
        return self.params

    def getNumberParams(self):
        return int(self._X_act) * self.n_params

    def _calcNumberParams(self):
        self.n_params = int(self.dim * self.rank)

    def setCovariance(self, cov):
        """ makes lowrank approximation of cov """
        assert cov.shape[0]==self.dim, 'Dimension mismatch.'
        S, U = la.eigh(cov)
        U = U[:,::-1]
        S = S[::-1]
        _X = U[:, :self.rank] * S[sp.newaxis, :self.rank]
        self.X = _X

    #####################
    # Cached
    #####################
    @cached('covar_base')
    def K(self):
        return sp.dot(self.X,self.X.T)

    @cached('covar_base')
    def Xgrad(self,i):
        Xgrad = sp.zeros(self.getNumberParams())
        Xgrad[i] = 1
        return sp.reshape(Xgrad, (self.dim, self.rank), order='F')

    @cached('covar_base')
    def K_grad_i(self,i):
        if not self._X_act:
            raise ValueError("Trying to retrieve the gradient over a "
                             "parameter that is inactive.")

        R  = sp.dot(self.X,self.Xgrad(i).T)
        R += R.T
        return R

    ####################
    # TODO: Interpretable Params
    ####################


if __name__ == '__main__':

    sp.random.seed(1)
    n = 4
    rank = 2
    cov = LowRankCov(n, rank)
    cov.setRandomParams()
    print cov.K()
    print cov.K_grad_i(0)



"""Datasets for LMM tests"""
import scipy as SP
import glob
import re
import os

filetype = ".txt"
#filetype = ".txt.gz"

def load(directory):
    stringpattern = "(.*)"+filetype
    pattern=re.compile(stringpattern)
    FL = glob.glob(os.path.join(directory,'*'+filetype))
    RV = {}
    for fn in FL:
        fn_ = os.path.split(fn)[-1] #only keep the filename
        name = pattern.match(fn_).group(1)
        RV[name] = SP.loadtxt(fn)
    return RV

def dump(R,directory):
    for r in R.keys():
        fn = os.path.join(directory,r+filetype)
        SP.savetxt(fn,R[r])
    
    



"""LMM testing code"""
import unittest
import scipy as SP
import pdb
import sys
import limix


class Acovar_test(object):
    """abstract test class for covars"""

    def test_grad(self):
        """test analytical gradient"""
        RV = self.C.check_covariance_Kgrad_theta(self.C)
        self.assertTrue(RV)

    # def test_hess(self):
    #     """test analytical hessian"""
    #     D2=SP.zeros((self.n_params,self.n_params))
    #     for i in range(self.n_params):
    #         for j in range(self.n_params):
    #             D2[i,j]=((self.C.Khess_param(i,j)-self.C.Khess_param_num(self.C,i,j))**2).max()
    #     RV=D2.max()<1E-6
    #     self.assertTrue(RV)



import sys
sys.path.insert(0,'./../debug.darwin/interfaces/python')
import limix
import scipy as SP
import data
import limix.deprecated.modules.varianceDecomposition as VAR
import pdb

if __name__ == '__main__':
    dir_name = './varDecomp/varDecomp'
    D = data.load(dir_name)
    S = D['X'].shape[1]
    P = D['Y'].shape[1]
    Y = D['Y']
    X = D['X']
    Y = Y[0:10]
    X = X[0:10]

    N = Y.shape[0]
    Kg = SP.dot(X,X.T)
    Kg = Kg/Kg.diagonal().mean()


    K0 = SP.eye(Y.shape[0])
    X0 = SP.randn(Y.shape[0],3)
    covar1 = limix.CCovLinearISO(3)
    covar1.setParams(SP.array([1.0]))
    covar1.setX(X0)
    covar2 = limix.CFixedCF(K0)
    covar2.setParams(SP.array([1.0]))

    covar=limix.CSumCF()
    covar.addCovariance(covar2)
    covar.addCovariance(covar1)

    ll  = limix.CLikNormalIso()
    gp=limix.CGPbase(covar,ll)

    hyperparams = limix.CGPHyperParams()
    hyperparams['covar'] = SP.array([1.0,1.0])
    hyperparams['lik'] = SP.array([0.1])
    hyperparams['X']   = X0

    gp.setY(Y)
    gp.setX(X0)
    gp.setParams(hyperparams)



#! /usr/bin/env python
# Copyright(c) 2014, The mtSet developers (Francesco Paolo Casale, Barbara Rakitsch, Oliver Stegle)
# All rights reserved.

from limix.mtSet.core.postprocessCore import postprocess
from optparse import OptionParser

def entry_point():

    parser = OptionParser()
    parser.add_option("--resdir", dest='resdir', type=str, default=None)
    parser.add_option("--outfile", dest='outfile', type=str, default=None)
    parser.add_option("--manhattan_plot", dest='manhattan',action="store_true",default=False)
    parser.add_option("--tol", dest='tol', type=float, default=4e-3)
    (options, args) = parser.parse_args()
    postprocess(options)



#! /usr/bin/env python
# Copyright(c) 2014, The mtSet developers (Francesco Paolo Casale, Barbara Rakitsch, Oliver Stegle)
# All rights reserved.

import sys
import os
from limix.mtSet.core.analyzeCore import analyze
from optparse import OptionParser

def entry_point():
    parser = OptionParser()
    parser.add_option("--bfile", dest='bfile', type=str, default=None)
    parser.add_option("--cfile", dest='cfile', type=str, default=None)
    parser.add_option("--pfile", dest='pfile', type=str, default=None)
    parser.add_option("--nfile", dest='nfile', type=str, default=None)
    parser.add_option("--wfile", dest='wfile', type=str, default=None)
    parser.add_option("--ffile", dest='ffile', type=str, default=None)
    parser.add_option("--resdir", dest='resdir', type=str, default=None)
    parser.add_option("--trait_idx",dest='trait_idx',type=str, default=None)

    parser.add_option("--rank_r",dest='rank_r',type=int, default=1)
    parser.add_option("--colCovarType_r",dest='colCovarType_r',type=str, default='lowrank')

    # start window, end window and permutations
    parser.add_option("--minSnps", dest='minSnps', type=int, default=None)
    parser.add_option("--start_wnd", dest='i0', type=int, default=None)
    parser.add_option("--end_wnd", dest='i1', type=int, default=None)
    parser.add_option("--perm", dest='perm_i', type=int, default=None)

    (options, args) = parser.parse_args()

    analyze(options)



'''
Created on Feb 20, 2013

@author: johannes
'''

import scipy.linalg as LA
import scipy as SP
import lmm_fast
import limix.deprecated as dlimix

class BLUP(object):
    '''
    classdocs
    '''
    def __init__(self):
        '''
        Constructor
        '''
    def reset(self):
        self = BLUP()

    def fit(self, XTrain=None, yTrain=None, KTrain=None, delta=None, folds=0):
        self.nTrain = yTrain.shape[0]
        self.yTrain = yTrain
        if KTrain is not None:
            self.kernel = KTrain
        self.S, self.U = LA.eigh(self.kernel+SP.eye(self.nTrain)*1e-8)
        self.Uy = SP.dot(self.U.T, yTrain)
        self.Uone = SP.dot(self.U.T, SP.ones_like(yTrain))
        if delta is None:
            # Initialize cross-learning of delta
            if folds > 0:
                self.ldelta = self.crossvalidate_delta(folds)
            else:
                self.ldelta = dlimix.optdelta(self.Uy[:, 0], self.Uone, self.S,
                                             100000, -5.0, 5.0)
            self.delta = SP.exp(self.ldelta)
        else:
            self.delta = delta
            self.ldelta = SP.log(delta)
        self.cov = LA.inv((self.kernel + SP.eye(self.nTrain)*self.delta))

    def update_delta(self, mean):
        Umean = SP.dot(self.U.T, mean)
        self.ldelta = dlimix.optdelta(self.Uy[:, 0], Umean, self.S, 100,
                                     self.ldelta-1., self.delta+1.)
        self.delta = SP.exp(self.ldelta)
        return self.delta

    def crossvalidate_delta(self, folds):
        import utils
        cv_scheme = utils.crossValidationScheme(folds, self.nTrain)
        ldeltas = SP.arange(-3, -1.5, .01)
        Ss = []
        Us = []
        Uys = []
        UCs = []
        err = 0.0
        errs = []
        for ldelta in ldeltas:
            for test_set in cv_scheme:
                train_set = ~test_set
                K_sub = self.kernel[SP.ix_(train_set, train_set)]
                K_cross = self.kernel[SP.ix_(~train_set, train_set)]
                # print LA.inv((K_sub + SP.eye(train_set.sum())*self.delta))
                Core = SP.dot(K_cross, LA.inv((K_sub + SP.eye(train_set.sum()) *
                                               SP.exp(ldelta))))
                diff = self.yTrain[test_set] -\
                    SP.dot(Core, self.yTrain[train_set])
                err += (diff**2).sum()/diff.size
                S, U = LA.eigh(self.kernel[SP.ix_(train_set, train_set)])
                Ss.append(S)
                Us.append(U)
                Uys.append(SP.dot(U.T, self.yTrain[train_set]))
                UCs.append(SP.dot(U.T, SP.ones_like(self.yTrain[train_set])))
            errs.append(err/len(cv_scheme))
            err = 0.0

        nll_scores = []
        for ldelta in ldeltas:
            # print 'ldelta equals', ldelta
            score = 0.0
            for i in xrange(len(cv_scheme)):
                score += lmm_fast.nLLeval(ldelta, (Uys[i])[:, 0], UCs[i], Ss[i])
            nll_scores.append(score/len(cv_scheme))
        print 'best ldelta found ll', ldeltas[SP.argmin(nll_scores)]
        return ldeltas[SP.argmin(errs)]

    def LL(self, mean):
        mean.reshape(-1, 1)
        res = self.yTrain - mean
        ll = SP.dot(SP.dot(res.T, self.cov), res)
        return ll

    def predict(self, XTest=None, k=None, mean=None):
        # k is cross covariance KTestTrain
        self.mean = mean
        if self.mean is None:
            self.mean = SP.ones_like(self.yTrain)*self.yTrain.mean()
        if k is None:
            k = self.kernel
        Core = SP.dot(k, LA.inv((self.kernel + SP.eye(self.nTrain)*self.delta)))
        return SP.dot(Core, self.yTrain-self.mean)



from __future__ import print_function
import os
from os.path import join
import sys
import importlib

PKG_NAME = 'limix'
VERSION  = '0.7.20'

WORKDIR = os.path.abspath(os.path.dirname(__file__))

def ERR(text, bold=False):
    text = '\033[91m' + text + '\033[0m'
    if bold:
        text = '\033[1m' + text
    return text

def query_yes_no(question, default="yes"):
    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is True for "yes" or False for "no".
    """
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = raw_input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")

def rreplace(s, old, new, occurrence):
    """ Replace last occurrence.
    """
    li = s.rsplit(old, occurrence)
    return new.join(li)

def try_import(pkgs):
    failed = []
    for pkg in pkgs:
        try:
            importlib.import_module(pkg)
        except ImportError:
            failed.append(pkg)
    return failed

def check_essentials():
    pkgs_failed = try_import(["setuptools", "cython", "numpy"])
    if len(pkgs_failed) > 0:

        plural = 's' if len(pkgs_failed) > 1 else ''
        pkgs_failed = [ERR(pf, True) for pf in pkgs_failed]

        s = rreplace(ERR(", ").join(pkgs_failed), ERR(", "), ERR(" and "), 1)

        msg  = ERR("Ops! We need the ")
        msg += s
        msg += ERR(" Python package{}".format(plural))
        msg += ERR(" installed before proceeding ")
        msg += ERR("with the {} installation.".format(PKG_NAME.upper()))
        print(msg)
        sys.exit(1)

check_essentials()

from setuptools import find_packages
from setuptools import setup
from setuptools.extension import Extension
from Cython.Build import cythonize
from Cython.Distutils import build_ext

def _check_gcc_cpp11(cc_name):
    import subprocess
    try:
        cmd = cc_name + ' -E -dM -std=c++11 -x c++ /dev/null > /dev/null'
        subprocess.check_call(cmd, shell=True)
    except subprocess.CalledProcessError:
        return False
    return True

class build_ext_subclass(build_ext):
    def build_extensions(self):
        if len(self.compiler.compiler) > 0:
            cc_name = self.compiler.compiler[0]
            stdcpp = '-std=c++11'
            if 'gcc' in cc_name and not _check_gcc_cpp11(cc_name):
                stdcpp = '-std=c++0x'
            for e in self.extensions:
                e.extra_compile_args.append(stdcpp)
        build_ext.build_extensions(self)

def globr(root, pattern):
    import fnmatch

    matches = []
    for root, _, filenames in os.walk(root):
        for filename in fnmatch.filter(filenames, pattern):
            matches.append(os.path.join(root, filename))

    return matches

def mac_workaround():
    import platform
    from distutils import sysconfig

    conf_vars = sysconfig.get_config_vars()
    vers = platform.mac_ver()[0].split('.')
    if len(vers) == 3:
        conf_vars['MACOSX_DEPLOYMENT_TARGET'] =\
            vers[0] + '.' + vers[1]
    else:
        conf_vars['MACOSX_DEPLOYMENT_TARGET'] = platform.mac_ver()[0]

def nlopt_files():
    src = open(join(WORKDIR, 'External', 'nlopt_src.files')).readlines()
    src = [join(WORKDIR, 'External', 'nlopt', s).strip() for s in src]
    hdr = globr(join(WORKDIR, 'External', 'nlopt'), '*/*.h')
    return (src, hdr)

def swig_opts():
    return ['-c++', '-outdir', join(WORKDIR, 'limix', 'deprecated'),
            '-I'+join(WORKDIR, 'src')]

def extra_compile_args():
    return ['-Wno-comment', '-Wno-unused-but-set-variable',
            '-Wno-overloaded-virtual', '-Wno-uninitialized',
            '-Wno-delete-non-virtual-dtor', '-Wunused-variable']

def extra_link_args():
    return []

def core_extension(reswig):
    import numpy as np

    (src, hdr) = nlopt_files()
    src.extend(globr(join(WORKDIR, 'src', 'limix'), '*.cpp'))
    hdr.extend(globr(join(WORKDIR, 'src', 'limix'), '*.h'))

    incl = ['src', 'External', join('External', 'nlopt')]
    incl = [join(WORKDIR, i) for i in incl]
    folder = join(WORKDIR, 'External', 'nlopt')
    incl += [join(folder, f) for f in os.listdir(folder)]
    incl = [i for i in incl if os.path.isdir(i)]
    incl.extend([np.get_include()])

    wrap_file = join(WORKDIR, 'src', 'interfaces', 'python', 'limix_wrap.cpp')
    i_file = join(WORKDIR, 'src', 'interfaces', 'python', 'limix.i')

    if os.path.exists(wrap_file):
        src.append(wrap_file)
    else:
        src.append(i_file)

    depends = src + hdr

    ext = Extension('limix.deprecated._core', src,
                    include_dirs=incl,
                    extra_compile_args=extra_compile_args(),
                    extra_link_args=extra_link_args(),
                    swig_opts=swig_opts(),
                    depends=depends)

    return ext

def ensemble_extension():
    import numpy as np

    src = [join(WORKDIR, 'cython', 'lmm_forest', 'SplittingCore.pyx')]
    incl = [join(WORKDIR, 'External'), np.get_include()]
    depends = src
    ext = Extension('limix.ensemble.SplittingCore', src,
                    language='c++',
                    include_dirs=incl,
                    extra_compile_args=extra_compile_args(),
                    extra_link_args=extra_link_args(),
                    depends=depends)
    return cythonize(ext)

def write_version():
    cnt = """
# THIS FILE IS GENERATED FROM %(package_name)s SETUP.PY
version = '%(version)s'
"""
    filename = os.path.join(PKG_NAME, 'version.py')
    a = open(filename, 'w')
    try:
        a.write(cnt % {'version': VERSION,
                       'package_name': PKG_NAME.upper()})
    finally:
        a.close()

def get_test_suite():
    from unittest import TestLoader
    return TestLoader().discover(PKG_NAME)

def setup_package(reswig, yes):
    if sys.platform == 'darwin':
        mac_workaround()

    install_requires = ["sklearn", "pandas"]
    setup_requires = []

    # These are problematic packages (i.e., C/Fortran dependencies) to
    # install from pypi so we leave the option to the user for doing so.
    problematic_pkgs = ["scipy", "h5py", "matplotlib"]

    pkgs_failed = try_import(problematic_pkgs)
    if len(pkgs_failed) > 0:
        pkgs_failed = [ERR(pf, True) for pf in pkgs_failed]
        fpkgs = rreplace(ERR(", ").join(pkgs_failed), ERR(", "),
                         ERR(" and "), 1)
        plural = 's' if len(pkgs_failed) > 1 else ''
        msg  = ERR("The ") + fpkgs + ERR(" Python package%s " % plural)
        plural = 'are' if len(pkgs_failed) > 1 else 'is'
        msg += ERR("%s needed by " % plural)
        msg += ERR(PKG_NAME.upper() + " but could not be found in your ")
        msg += ERR("system.")
        print(msg)
        print("We recommend their installation to be done through "
              "package managers (e.g., conda, canopy, apt-get, yum, brew) "
              "but we can also try to install them right now.")

        if not yes:
            yes = query_yes_no("Do you want to try to install them right now?",
                               "no")
        if yes:
            install_requires.extend(problematic_pkgs)
        else:
            print("Good choice! Their installation tend to be painless via "
                  "package managers.")
            sys.exit(0)

    src_path = os.path.dirname(os.path.abspath(sys.argv[0]))
    old_path = os.getcwd()
    os.chdir(src_path)
    sys.path.insert(0, src_path)

    write_version()

    metadata = dict(
        name=PKG_NAME,
        description="A flexible and fast mixed model "+
                    "toolbox written in C++/python",
        long_description=open(join(WORKDIR, 'README'), 'r').read(),
        keywords='linear mixed models, GWAS, QTL, ' +
                 'Variance component modelling',
        maintainer="Limix Developers",
        author="Christoph Lippert, Paolo Casale, Oliver Stegle",
        author_email="stegle@ebi.ac.uk",
        maintainer_email="stegle@ebi.ac.uk",
        version=VERSION,
        test_suite='setup.get_test_suite',
        packages=find_packages(exclude=['tests', 'test', 'test_limix*']),
        license="BSD",
        url='http://pmbio.github.io/limix/',
        install_requires=install_requires,
        setup_requires=setup_requires,
        zip_safe=False,
        ext_modules=[core_extension(reswig)] + ensemble_extension(),
        cmdclass=dict(build_ext=build_ext_subclass),
        entry_points={
            'console_scripts':[
                'limix_runner=limix.scripts.limix_runner:entry_point',
                'mtSet_postprocess=limix.scripts.mtSet_postprocess:entry_point',
                'mtSet_preprocess=limix.scripts.mtSet_preprocess:entry_point',
                'mtSet_simPheno=limix.scripts.mtSet_simPheno:entry_point',
                'mtSet_analyze=limix.scripts.mtSet_analyze:entry_point',
                'limix_converter=limix.scripts.limix_converter:entry_point'
            ]
        }
    )

    try:
        from distutils.command.bdist_conda import CondaDistribution
        metadata['distclass'] = CondaDistribution
        metadata['conda_buildnum'] = 1
        metadata['conda_features'] = ['mkl']
    except ImportError:
        pass

    # http://stackoverflow.com/a/29634231
    import distutils.sysconfig
    cfg_vars = distutils.sysconfig.get_config_vars()
    for key, value in cfg_vars.items():
        if type(value) == str:
            cfg_vars[key] = value.replace("-Wstrict-prototypes", "")

    try:
        setup(**metadata)
    finally:
        del sys.path[0]
        os.chdir(old_path)

if __name__ == '__main__':
    reswig = False
    if "--reswig" in sys.argv:
        reswig = True
        sys.argv.remove("--reswig")

    yes = False
    if "--yes" in sys.argv:
        yes = True
        sys.argv.remove("--yes")
    setup_package(reswig, yes)



import inspect
import os
import pprint
import re
from time import time
from limix.utils.util_functions import smartSum


# HOW TO USE: Please, look at the end of the file for an example.

def std_obj_repr(obj):
    return "<%s instance at %s>" % (obj.__class__.__name__, hex(id(obj)))

class Cached(object):

    def __init__(self):
        self._cache_groups = dict()
        self._diff_running = False

        for (_, m) in inspect.getmembers(type(self),
                                         predicate=lambda p:
                                         (inspect.ismethod or
                                         inspect.isdatadescriptor)):

            if hasattr(m, 'fget'):
                f = m.fget
            elif hasattr(m, 'im_func'):
                f = m.im_func
            else:
                continue

            fv = f.func_code.co_freevars

            try:
                closure = f.func_closure
            except AttributeError:
                continue

            if closure is None:
                continue

            vs = dict(zip(fv, (c.cell_contents for c in closure)))

            # this is used to make sure we are in the right function
            # i'm not proud of that, by the way
            if '_cache_identifier_pj97YCjgnp' not in vs:
                continue

            try:
                groups = vs['groups']
                method_name = re.match('^_cache_(.+)$',
                                       vs['cache_var_name']).group(1)
            except KeyError:
                continue

            for g in groups:
                if g not in self._cache_groups:
                    self._cache_groups[g] = []
                self._cache_groups[g].append(method_name)

            setattr(self, '_cache_' + method_name, None)
            setattr(self, '_cached_' + method_name, False)
            setattr(self, '_cached_args_' + method_name, dict())

        self._reset_profiler()

    def _reset_profiler(self):
        self.time = {}
        self.time_in = {}
        self.counter = {}
        cmethods = self._registered_methods()
        for m in cmethods:
            self.time[m] = 0
            self.time_in[m] = 0
            self.counter[m] = 0

    def diff(self, func, *args, **kwargs):
        if self._diff_running:
            return

        self._diff_running = True

        pp = pprint.PrettyPrinter(indent=4)

        print '*** Cache diff ***'

        self._print_groups()

        print '-- Cached methods --'
        cmethods = self._registered_methods()
        pp.pprint(cmethods)

        caches_before = self._get_caches()
        func(*args, **kwargs)
        caches_after = self._get_caches()

        print '-- Difference between caches -- '
        for cm in cmethods:

            try:
                changed = caches_before[cm] is not caches_after[cm]
            except:
                import ipdb; ipdb.set_trace()

            print cm+':', changed

            #if caches_before[cm] is not caches_after[cm]:
            #    print '%s: %s --> %s' % (cm, std_obj_repr(caches_before[cm]),
            #                                 std_obj_repr(caches_after[cm]))
        print '*** End ***'
        self._diff_running = False

    def _get_caches(self):
        methods = self._registered_methods()
        caches = dict()
        for m in methods:
           caches[m] = getattr(self, '_cache_' + m)
        return caches

    def _get_cached_methods(self):
        methods = self._registered_methods()
        cmethods = []
        for m in methods:
            # if hasattr(self, '_cache_' + m) and getattr(self, '_cached_' + m):
            cmethods.append(m)
        return cmethods

    def _registered_methods(self):
        methods = set()
        for g in self._cache_groups.values():
            for m in g:
                methods.add(m)
        return list(methods)

    def _print_groups(self):
        print '-- Cache groups and the respective registered methods --'
        pp = pprint.PrettyPrinter(indent=4)
        pp.pprint(self._cache_groups)

    def clear_cache(self, *cache_groups):
        assert hasattr(self, '_cache_groups'), "Cache system failed because "\
                                   + "you did not call Cached's constructor."

        for g in cache_groups:
            if g not in self._cache_groups:
                raise Exception('Cache group %s does not exist.' % g)

            for method_name in self._cache_groups[g]:
                setattr(self, '_cache_' + method_name, None)
                setattr(self, '_cached_' + method_name, False)
                setattr(self, '_cached_args_' + method_name, dict())

    def fill_cache(self, method_name, value):
        if method_name not in self._registered_methods():
           raise ValueError("The method %s is not registered in this cache."
                            % method_name)
        setattr(self, '_cache_' + method_name, value)
        setattr(self, '_cached_' + method_name, True)

    def get_time_profile(self, reduced=True):
        if reduced:
            # reduce time to fields taht have been caculated at least once
            rv = {k:v for k,v in self.time.items() if self.counter[k]>0}
        else:
            rv = self.time
        return rv

    def get_timein_profile(self, reduced=True):
        if reduced:
            # reduce time to fields taht have been caculated at least once
            rv = {k:v for k,v in self.time_in.items() if self.counter[k]>0}
        else:
            rv = self.intime
        return rv

    def get_counter_profile(self, reduced=True):
        if reduced:
            # reduce time to fields taht have been caculated at least once
            rv = {k:v for k,v in self.counter.items() if v>0}
        else:
            rv = self.counter
        return rv

    def _profile(self, show=False, fs=10, rot=90, reduced=True):
        import pylab as pl
        import scipy as sp
        t_dict = self.get_time_profile(reduced=reduced)
        c_dict = self.get_counter_profile(reduced=reduced)
        labels = t_dict.keys()
        t = sp.array([t_dict[m] for m in labels])
        c = sp.array([c_dict[m] for m in labels])
        x = sp.arange(t.shape[0])
        plt = pl.subplot(211)
        pl.bar(x,t)
        plt.set_xticks(x + 0.5)
        #plt.set_xticklabels(labels, fontsize=fs, rotation=rot)
        plt = pl.subplot(212)
        pl.bar(x,c)
        plt.set_xticks(x + 0.5)
        plt.set_xticklabels(labels, fontsize=fs, rotation=rot)
        if show:
            pl.tight_layout()
            pl.show()


# This decorator works both with arguments or without
# (i.e., @cached, @cached("group_name"), @cached(["group_name_A",
#       "group_name_B"]), or @cached(exclude=['param1', 'param2', ...]))
def cached(*args, **kwargs):

    deco_without_arg = len(args) == 1 and inspect.isfunction(args[0])

    # cached_args stores the argument names and argument values with which
    # the method was called in
    # cached_args = dict()
    filter_out = []
    _groups = ['default']
    # This is a hacky thing so that one can know that a method_wrapper
    # object, defined bellow, really came from here.
    _cache_identifier_pj97YCjgnp = [False]

    if not deco_without_arg:
        if len(args) > 0:
            if type(args[0]) is list or type(args[0]) is tuple:
                _groups += list(args[0])
            else:
                _groups.append(args[0])

        if len(kwargs) == 1:
            assert 'exclude' in kwargs, ("'exclude' is the only keyword "
                                         "allowed here.")
            filter_out = kwargs['exclude']
    else:
        _groups.append(args[0].__name__)

    def real_cached(method):

        # dont look at this miserable hacky variable
        _cache_identifier_pj97YCjgnp[0] = True

        cache_var_name = '_cache_' + method.__name__
        valid_var_name = '_cached_' + method.__name__
        cached_args_name = '_cached_args_' + method.__name__
        groups = _groups

        debug = int(os.getenv('LIMIX_DEBUG', 0))
        if debug:
            return method

        def method_wrapper(self, *args, **kwargs):

            t0 = time()

            # dont look at this miserable hacky variable
            _cache_identifier_pj97YCjgnp[0] = True
            # this is meant to insert groups in this scope
            groups

            (argnames, argvalues) = _fetch_argnames_argvalues(method, args,
                                                              kwargs)

            t = zip(argnames, argvalues)
            provided_args = dict((x, y) for x, y in t)

            for f in filter_out:
                del provided_args[f]

            if getattr(self, valid_var_name, False) is False or\
                    provided_args != getattr(self, cached_args_name, dict()):
                t0in = time()
                result = method(self, *args, **kwargs)
                try:
                    self.time_in[method.__name__] += time() - t0in
                except:
                    import ipdb; ipdb.set_trace()
                setattr(self, cache_var_name, result)
                setattr(self, valid_var_name, True)
                if hasattr(self, cached_args_name):
                    getattr(self, cached_args_name).clear()
                    getattr(self, cached_args_name).update(provided_args)
                else:
                    setattr(self, cached_args_name, dict())
                self.counter[method.__name__] += 1.

            self.time[method.__name__] += time() - t0

            return getattr(self, cache_var_name)

        return method_wrapper

    if deco_without_arg:
        return real_cached(args[0])

    return real_cached


# ------------------------ INTERNAL USE ONLY ------------------------


def _map_args_kwargs_to_argvalues(args, kwargs, argnames, defaults):

    argvalues = [None] * len(argnames)
    for i in xrange(len(defaults)):
        argvalues[len(argnames) - len(defaults) + i] = defaults[i]

    for i in xrange(len(args)):
        argvalues[i] = args[i]

    for kw in kwargs:
        index = argnames.index(kw)
        argvalues[index] = kwargs[kw]

    return argvalues


# This function retrieves and organizes the method argument names and
# their values at the moment of the call. For this task,
# I need to both inspect the method (looking for default values) and
# the actual params passed by the user. Combining these two source of
# information I can return (argnames, argvalues) with all the argument names
# and argument values.
def _fetch_argnames_argvalues(method, args, kwargs):
    # argnames = inspect.getargspec(method)[0]
    nargs = method.func_code.co_argcount
    names = method.func_code.co_varnames
    argnames = list(names[:nargs])

    if len(argnames) == 1:
        return ([],[])

    # assert argnames[0] == 'self'
    # del argnames[argnames.index('self')]
    del argnames[0]

    # defaults = inspect.getargspec(method)[3]
    defaults = method.func_defaults
    if defaults is None:
        defaults = []
    argvalues = _map_args_kwargs_to_argvalues(args, kwargs, argnames, defaults)

    return (argnames, argvalues)


if __name__ == '__main__':
    class Test(Cached):
        @cached
        def foo1(self, par1, par2=None):
            print 'called',
            return 5

        @cached(exclude=['par2'])
        def foo2(self, par1, par2):
            print 'called',
            return 5

        @cached("A")
        def foo3(self, par1):
            print 'called',
            return 5

        @cached(["A", "B"])
        def foo4(self, par1):
            print 'called',
            return 5

        @cached(["B"])
        def foo5(self, par1):
            print 'called',
            return 5

    test = Test()
    print '\ntest.foo1(2, par2=0)',
    test.foo1(2, par2=0)
    print '\ntest.foo1(2, par2=1)',
    test.foo1(2, par2=1)
    print '\ntest.foo1(2, par2=0)',
    test.foo1(2, par2=0)
    print '\ntest.foo1(2, par2=3)',
    test.foo1(2, par2=3)
    print '\ntest.foo1(2, par2=3)',
    test.foo1(2, par2=3)
    print '\ntest.foo1(2)',
    test.foo1(2)
    print '\ntest.foo1(3)',
    test.foo1(3)
    print '\ntest.foo1(3)',
    test.foo1(3)

    print ''

    print '\ntest.foo2(1, 2)',
    test.foo2(1, 2)
    print '\ntest.foo2(1, 2)',
    test.foo2(1, 2)
    print '\ntest.foo2(1, 3)',
    test.foo2(1, 3)
    print '\ntest.foo2(2, 3)',
    test.foo2(2, 3)
    print '\ntest.foo2(2, 3)',
    test.foo2(2, 3)

    print ''
    print '\ntest.foo3(1)',
    test.foo3(1)
    print '\ntest.foo3(1)',
    test.foo3(1)
    print '\ntest.foo4(1)',
    test.foo4(1)
    print '\ntest.foo4(1)',
    test.foo4(1)
    print '\ntest.foo5(1)',
    test.foo5(1)
    print '\ntest.foo5(1)',
    test.foo5(1)
    print '\nClearing cache group A',
    test.clear_cache("A")
    print '\ntest.foo3(1)',

    test.foo3(1)
    print '\ntest.foo3(1)',
    test.foo3(1)
    print '\ntest.foo4(1)',
    test.foo4(1)
    print '\ntest.foo4(1)',
    test.foo4(1)
    print '\ntest.foo5(1)',
    test.foo5(1)
    print '\ntest.foo5(1)',
    test.foo5(1)
    print '\nClearing cache group B',
    test.clear_cache("B")
    print '\ntest.foo3(1)',
    test.foo3(1)
    print '\ntest.foo3(1)',
    test.foo3(1)
    print '\ntest.foo4(1)',
    test.foo4(1)
    print '\ntest.foo4(1)',
    test.foo4(1)
    print '\ntest.foo5(1)',
    test.foo5(1)
    print '\ntest.foo5(1)',
    test.foo5(1)
    print '\nClearing cache group default',
    test.clear_cache("default")
    print '\ntest.foo3(1)',
    test.foo3(1)
    print '\ntest.foo3(1)',
    test.foo3(1)
    print '\ntest.foo4(1)',
    test.foo4(1)
    print '\ntest.foo4(1)',
    test.foo4(1)
    print '\ntest.foo5(1)',
    test.foo5(1)
    print '\ntest.foo5(1)',
    test.foo5(1)



import sys
sys.path.insert(0,'./../../..')
from limix.core.cobj import * 
from limix.utils.preprocess import regressOut
from limix.core.mean.mean import compute_X1KX2
from limix.core.mean.mean import compute_XYA
import  limix.utils.psd_solve as psd_solve
import numpy as np
		    
import scipy.linalg as la
import copy
import pdb


class Mean(cObject):

    def __init__(self,Y):
        """ init data term """
        self.Y = Y
        self.clearFixedEffect()

    #########################################
    # Properties 
    #########################################

    @property
    def len(self):
        return len(self.A)

    @property
    def shape(self):
        return self.Y.shape

    def getDimensions(self):
        """ get phenotype dimensions """
        return self.shape

    @property
    def N(self):
        return self.Y.shape[0]

    @property
    def P(self):
        return self.Y.shape[1]
    
    @property
    def Lr(self):
        return self._Lr

    @property
    def Lc(self):
        return self._Lc

    @property
    def d(self):
        return self._d

    @property
    def D(self):
        return np.reshape(self.d,(self.N,self.P), order='F')

    @property
    def LRLdiag(self):
        return self._LRLdiag

    @property
    def LCL(self):
        return self._LCL

    @property
    def dof(self, index=None):
        """The number of degrees of freedom"""
        if index is None:
            dof = 0
            for i in xrange(self.len):
                dof += self.A[i].shape[0] * self.F[i].shape[1]
            return dof
        else:
            return self.A[index].shape[0] * self.F[index].shape[1]
    
    @property
    def dof_any(self):
        """The number of degrees of freedom for any effects terms"""
        return self.P * self.F_any.shape[1]
    
    @d.setter
    def d(self,value):
        """ set anisotropic scaling """
        assert value.shape[0]==self._P*self._N, 'd dimension mismatch'
        self._d = value
        self.clear_cache()
    #########################################
    # Setters 
    #########################################

    def clearFixedEffect(self):
        """ erase all fixed effects """
        self.A = []
        self.F = []
        self.F_any = np.zeros((self.N,0))
        self.clear_cache()

    def addFixedEffect(self,F=None,A=None,index=None):
        """
        set sample and trait designs
        F:      NxK sample design
        A:      LxP sample design
        fast_computations:   False deactivates the fast computations for any and common effects (for debugging)
        """
        if F is None:
            F = np.ones((self.N,1))
        else:
            assert F.shape[0]==self.N, "F dimension mismatch"

        if ((A is None) or ( (A.shape == (self.P,self.P)) and (A==np.eye(self.P)).all() )):
            #case any effect
            self.F_any = np.hstack((self.F_any,F))
        elif (index is not None) and  ((A==self.A[index]).all()):
            #case common effect
            self.F[index] = np.hstack((self.F_index,F))
        else:
            #case general A
            assert A.shape[1]==self.P, "A dimension mismatch"
            self.F.append(F)
            self.A.append(A)

        self.clear_cache()


    @Lr.setter
    def Lr(self,value):
        """ set row rotation """
        assert value.shape==(self.N, self.N), 'dimension mismatch'
        self._Lr = value
        self.clear_cache()

    @Lc.setter
    def Lc(self,value):
        """ set col rotation """
        assert value.shape==(self.P, self.P), 'dimension mismatch'
        self._Lc = value
        self.clear_cache()

    @d.setter
    def d(self,value):
        """ set anisotropic scaling """
        assert value.shape[0]==self.P*self.N, 'd dimension mismatch'
        self._d = value
        self.clear_cache()

    @LRLdiag.setter
    def LRLdiag(self,value):
        """ set LRLdiag """
        self._LRLdiag = value
        self.clear_cache()

    @LCL.setter
    def LCL(self,value):
        """ set LCL """
        self._LCL = value
        self.clear_cache()

    #########################################
    # Getters (caching)
    #########################################

    @property
    def Astar(self):
        RV = []
        for term_i in range(self.len):
            RV.append(np.dot(self.A[term_i],self.Lc.T))
        return RV 

    @property
    def Fstar(self):
        RV = []
        for term_i in range(self.len):
            RV.append(np.dot(self.Lr,self.F[term_i]))
        return RV 

    @property
    def Astar_any(self):
        return np.eye(self.P) 
    
    @property
    def Fstar_any(self):
        return np.dot(self.Lr,self.F_any)

    def Ystar1(self):
        return np.dot(self.Lr,self.Y)

    def Ystar(self):
        return np.dot(self.Ystar1(),self.Lc.T)

    def Yhat(self):
        return self.D*self.Ystar()

    @property
    def Areml_solver(self):
        return psd_solve.PsdSolverKron(A = self.XKX(), A_any = self.XanyKXany(), AA_any = self.XanyKX())

    def beta_hat(self):
        """compute ML beta"""
        XKY = self.XKY()
        XanyKY = self.XanyKY()
        beta_hat, beta_hat_any = self.Areml_solver.solve(b_any=XanyKY,b=XKY,check_finite=True)
        return beta_hat, beta_hat_any

    def var_total(self):
        return (self.Yhat()*self.Ystar()).sum()
        

    def var_explained(self, identity_trick = False):
        XKY = self.XKY()
        XanyKY = self.XanyKY()
        beta_hat, beta_hat_any = self.Areml_solver.solve(b_any=XanyKY,b=XKY,check_finite=True)
        var_explained = (XKY*beta_hat).sum() + (XanyKY*beta_hat_any).sum()
        return var_explained, beta_hat, beta_hat_any

    ###############################################
    # Other getters with no caching, should not they have caching somehow?
    ###############################################


    def XanyKXany(self):
        """
        compute self covariance for any
        """
        result = np.empty((self.P,self.F_any.shape[1],self.F_any.shape[1]), order='C')
        for p in xrange(self.P):
            X1D = self.Fstar_any * self.D[:,p:p+1]
            X1X2 = X1D.T.dot(self.Fstar_any)
            result[p] = X1X2
        return result


    def XanyKX(self):
        """
        compute cross covariance for any and rest
        """
        result = np.empty((self.P,self.F_any.shape[1],self.dof), order='C')
        #This is trivially parallelizable:
        for p in xrange(self.P):
            FanyD = self.Fstar_any * self.D[:,p:p+1]
            start = 0
            #This is trivially parallelizable:
            for term in xrange(self.len):
                stop = start + self.F[term].shape[1]*self.A[term].shape[0]
                result[p,:,start:stop] = self.XanyKX2_single_p_single_term(p=p, F1=FanyD, F2=self.Fstar[term], A2=self.Astar[term])
                start = stop
        return result

    def XKX(self):
        """
        compute self covariance for rest
        """
        cov_beta = np.zeros((self.dof,self.dof))
        start_row = 0
        #This is trivially parallelizable:
        for term1 in xrange(self.len):
            stop_row = start_row + self.A[term1].shape[0] * self.F[term1].shape[1]
            start_col = start_row
            #This is trivially parallelizable:
            for term2 in xrange(term1,self.len):
                stop_col = start_col + self.A[term2].shape[0] * self.F[term2].shape[1]
                cov_beta[start_row:stop_row, start_col:stop_col] = compute_X1KX2(Y=self.Ystar(), D=self.D, X1=self.Fstar[term1], X2=self.Fstar[term2], A1=self.Astar[term1], A2=self.Astar[term2])
                if term1!=term2:
                    cov_beta[start_col:stop_col, start_row:stop_row] = cov_beta[n_weights1:stop_row, n_weights2:stop_col].T    
                start_col = stop_col
            start_row = stop_row
        return cov_beta
        

    def XanyKX2_single_p_single_term(self, p, F2, A2=None, F1=None):
            X1D = F1
            if F1 is None:
                self.Fstar_any * self.D[:,p:p+1]
            X1X2 = X1D.T.dot(F2)
            if A2 is not None:
                return np.kron(A2[:,p:p+1].T,X1X2)
            else:
                return X1X2

    def XanyKY(self):

        return compute_XYA(DY=self.Yhat(), X=self.Fstar_any, A=None).T

    def XKY(self):
        M = self.Yhat()
        XKY = np.zeros((self.dof))

        n_weights = 0
        for term in xrange(self.len):
            XKY_block = compute_XYA(DY=M, X=self.Fstar[term], A=self.Astar[term])
            XKY[n_weights:n_weights + self.A[term].shape[0] * self.F[term].shape[1]] = XKY_block.ravel(order='F')
            n_weights += self.A[term].shape[0] * self.F[term].shape[1]

        return XKY


    #########################################
    # Utility functions
    #########################################